
#include <iostream>
#include <assert.h>

#include "gpibc/gpib.h"
#include "gpibc/gco.hxx"
#include "gpibc/gdi.hxx"
#include "gpibc/gumi.hxx"
#include "gpib/gumi_user.hxx"
#include "gpib/gu_c.hxx"
#include "gpib/gu_d.hxx"

#include "gpib/gp_if.hxx"



  namespace Gif=INdepGpib100;
  namespace Umi=INdepUmi100;


  /*
   * controller program for the ISIM device simulator
   */

  #define GAD_CO  0x1e
  #define GAD_ISIM  0x3

  int failed( const Gif::GpibControl *pgc, const char *sz, int iLine=0 ) {
    if (pgc->isBad()) std::cerr << "bad " << sz << ' ' << std::dec << iLine << std::endl ;
    else std::cerr << "failed " << sz << ' ' << std::dec << iLine << ' ' << pgc->eErr() <<  std::endl ;
    return 2;
  }

  int run(Gif::GpibControl *pgc) {
      /* test the lcp */
      if (!pgc->open(true, GAD_CO)) return failed(pgc, "open");

      if (!pgc->ifc()) return failed(pgc, "ifc");

      if (!pgc->ren( true)) return failed(pgc,"ren");
      if (!pgc->ren( false)) return failed(pgc,"ren");
      if (!pgc->ren( true)) return failed(pgc, "ren");


      if (!pgc->clear(GAD_ISIM)) return failed(pgc, "clear");

      int cRun;
      for (cRun=1; cRun<2400; cRun++ ) {
	/* address listener */


	/* enable SRQ */
	if (!pgc->sendsz(GAD_ISIM, "STEN")) return failed(pgc, "send", __LINE__);
	if (!pgc->wait(Gif::E_SRQI, Umi::Time(10))) return failed(pgc, "wait", __LINE__);

        /* read the status byte */
	char ch;
	if (!pgc->readStatusByte(&ch, GAD_ISIM)) return failed(pgc, "spoll", __LINE__);
	std::cerr << std::hex << (int)((unsigned char)ch) << std::endl;

	/*
	 * read the ESR register
	 */
	if (!pgc->sendsz(GAD_ISIM, "*ESR?")) return failed(pgc, "send", __LINE__);

	
	char mpch[2048];
	unsigned c=sizeof(mpch);
	if (!pgc->receive(mpch, &c, GAD_ISIM)) return failed(pgc, "receive", __LINE__);

      }

      Gif::GpibCmds gc(GAD_CO);
      gc.cmdAdd(GPIB_UNL);
      gc.cmdAdd(GPIB_UNT);
      pgc->cmd( gc.pch(), gc.size(), Umi::Time(10,0));
      if (pgc->isFail()) return failed(pgc, "cmd");

      /* remote "power off" the similator */
      //if (!pgc->sendsz("POFF\n", GAD_ISIM, Umi::Time(4))) return failed(pgc, "send", __LINE__);


      if (!pgc->close()) return failed(pgc, "close");
      std::cout << cRun << " done " << std::endl;

      return 0;
    }


  int main(int argc, char **argv) {
    if (argc < 2) {
      std::cerr << "usage: " << argv[0] << " <interface>" << std::endl;
      return 2;
    }

    Gif::GpibInterfaceControl gi(argv[1]);
    if (!gi) {
      std::cerr << argv[0] << ": can't open " << argv[1] << ' ' << gi << std::endl;
      return 2;
    }
    return run(gi.pGuc());
  }
