
#include <iostream>
#include <assert.h>

#include "gpibc/gpib.h"
#include "gpibc/gco.hxx"
#include "gpibc/gdi.hxx"
#include "gpibc/gumi.hxx"
#include "gpib/gumi_user.hxx"
#include "gpib/gu_c.hxx"
#include "gpib/gu_d.hxx"

#include "gpib/gp_if.hxx"



  namespace Gif=INdepGpib100;
  namespace Umi=INdepUmi100;

  /*
   * GPIB local demo
   */
  int k195a(Gif::GpibControl *pgc);

  int main(int argc, char **argv) {
    if (argc < 2) {
      std::cerr << "usage: " << argv[0] << " <interface>" << std::endl;
      return 2;
    }

    Gif::GpibInterfaceControl gi(argv[1]);
    if (!gi) {
      std::cerr << argv[0] << ": can't open " << argv[1] << ' ' << gi << std::endl;
      return 2;
    }
    return k195a(gi.pGuc());				// application
  }

  /*
   * K195a test
   */
  #define GAD_CO  0x1e
  #define GAD_K195A  9


  int failed( const Gif::GpibControl *pgc, const char *sz, int iLine=0 ) {
    if (pgc->isBad()) std::cerr << "bad " << sz << ' ' << iLine << std::endl ;
    else std::cerr << "failed " << sz << ' ' << iLine << ' ' << pgc->eErr() <<  std::endl ;
    return 2;
  }

  int k195a(Gif::GpibControl *pgc) {
      /* test the lcp */
      if (!pgc->open( true, GAD_CO)) return failed(pgc, "open");

      if (!pgc->ifc()) return failed(pgc, "ifc");

      if (!pgc->ren( true)) return failed(pgc,"ren");
      if (!pgc->ren( false)) return failed(pgc,"ren");
      if (!pgc->ren( true)) return failed(pgc, "ren");

      //if (lm.cac(0, false)) return failed("cac=0");
      //if (lm.cac(0, true)) return failed("cac=1");

      if (!pgc->clear(GAD_K195A)) return failed(pgc, "clear");

      int cRun;
      //pgc->setLog(&std::cerr);
      for (cRun=0; cRun<10; cRun++ ) {

	/* send data */
	if (!pgc->sendsz(GAD_K195A, "F0R7Z0P0Q0X")) return failed(pgc, "send", __LINE__);

	/* receive data */
	char mpch[2048];
	unsigned c=sizeof(mpch);
	if (!pgc->receive(mpch, &c, GAD_K195A)) return failed(pgc, "receive", __LINE__); ////

	std::cout.write((char*)mpch, c);
	std::cout.flush();

	pgc->wait(Gif::E_SRQI, 0);


	if (1) {
	  /* sample into internal buffer */
	  if (!pgc->sendsz(GAD_K195A, "D  RUNX")) return failed(pgc, "send", __LINE__);

	  /* high speed sample until buffer full */
	  if (!pgc->sendsz(GAD_K195A, "S0M4X")) return failed(pgc, "send", __LINE__);

	  /* enable auto range */
	  if (!pgc->sendsz(GAD_K195A, "R0X")) return failed(pgc, "send", __LINE__);

	  /* clear buffer */
	  if (!pgc->sendsz(GAD_K195A, "Q0X")) return failed(pgc, "send", __LINE__);

	  /* no delay */
	  if (!pgc->sendsz(GAD_K195A, "Q01X")) return failed(pgc, "send", __LINE__);
        }

	  std::cout << "wait..." << std::endl;
	  if (!pgc->wait(Gif::E_SRQI, Umi::Time(10))) return failed(pgc, "wait", __LINE__);
	  std::cout << "done" << std::endl;

	if (1) {
	  /* read the status byte (we do not evaluate) */
	  char ch;
	  if (!pgc->readStatusByte(&ch, GAD_K195A)) return failed(pgc, "spoll", __LINE__);
	  std::cout << std::hex << (int)((unsigned char)ch) << std::endl;

	  /* read complete buffer on next transfer */
	  if (!pgc->sendsz(GAD_K195A, "B1X")) return failed(pgc, "send", __LINE__);

	  c=sizeof(mpch);
	  if (!pgc->receive(mpch, &c, GAD_K195A)) return failed(pgc, "receive", __LINE__);

	  std::cout.write((char*)mpch, 24) << std::endl;
	  std::cout << std::dec << c << std::endl;

	}
      }

      pgc->cmd("?_", 2, Umi::Time(1,0));
      if (pgc->isFail()) return failed(pgc, "cmd");

      std::cout << cRun << " done " << std::endl;
      if (!pgc->close()) return failed(pgc, "close");

      return 0;
    }

