
#ifndef _BASE_DELTA_HXX
#define _BASE_DELTA_HXX 1

/*
 *	delta.hxx
 *	delta and serial processing
 */

typedef int vers_t;
#define VERS_NIL -1		
#define VERS_FUTURE -2	
#define VERS_PAST -3

/*
 *	version pool 
 *
 *	Each version from the less recent (older) set is older (<) than
 *	any version from the more recent (current) set. In any set, versions
 *	are ordered (<,=,>). This way we have a complete order of all versions
 *	currently used.
 *
 *	Up to maxInUse sequentially ordered versions can exist in the system before 
 *	the oldest version must be returned. If a version is never returned (lost),
 *	an error condition (!canAlloc()) will result. 
 *
 *	Using the constraints above the version pool provides endless sequential numbering
 *	without overflow of the version number set.
 *
 *	Versions must always be compared (>,<) in the context of the version pool which
 *	emitted them. 
 *
 */
class VersPool {
	struct Set {
		int _i;			/* to be returned 	*/
		int _iLim;		/* max+1 	  	*/
		int _c;			/* in use	  	*/
	} _mpSet[2];			
	Set	*_psCur;		/* current (m/r) set	*/
	Set 	*psOther() { return (_psCur == _mpSet) ? _mpSet+1 : _mpSet ; }
public:
	VersPool(int cMaxInUse=1024);
	~VersPool();

	vers_t alloc();		/* new version, less recent than any vers ever got */
	void free(vers_t);	/* return version		*/
	int lt(vers_t, vers_t); /* true, iff left < right	*/
};

#endif
