

#ifndef _GP_IF_HXX_
#define _GP_IF_HXX_ 1


#include "gpib/gu_c.hxx"
#include "gpib/gu_d.hxx"
#include "gpib/gu_t.hxx"



#include <assert.h>

  /*
   * gp_if.hxx
   *
   * topmost GPIB Interface construction facility
   */

namespace INdepGpib100 {
  namespace Umi=INdepUmi100;

  class GpibInterface:public Failable {
    GpibControl *_pGuc;
    GpibDevice *_pGud;
    GpibTest *_pGut;
     

   public:
    GpibControl *pGuc() { return _pGuc; }
    GpibDevice *pGud() { return _pGud; }
    GpibTest *pGut() { return _pGut; }


    /* change ownership of allocated object */
    GpibControl *pGucRelease() { GpibControl *pGuc(_pGuc); _pGuc=0; return pGuc; }


    /*
     * szType = type of interface	(c)controller, (d)evice, t(est)
     * szMode = access domain 		(l)ocal, (k)ernel, (n)et
     * szArg =			l	memory address	(hex)
     *				k	dev,driver,chan triplet
     *				n	server,port
     */
    GpibInterface(Umi::PFF *mpPFF, const char *szType, const char *szMode, const char *szArg);
    ~GpibInterface();
  };



  inline const char *szMode(const char *sz) {
    return sz;
  }

  inline const char *szArg(const char *sz) {
    for (; *sz; sz++) {
      if (*sz == ':' && *(sz+1)) {
        return sz+1;
      }
    }
    return "";
  }

  inline const char *szField(const char *sz, int i) {
    if (!i) return sz;
    else i--;
    for (; *sz; sz++) {
      if (*sz == ':' && *(sz+1)) {
        if (!i) return sz+1;
	else i--;
      }
    }
    return "";
  }

  extern Umi::PFF mppfStd[];

  class GpibInterfaceAny:public GpibInterface {
   public:
    GpibInterfaceAny(const char *sz):GpibInterface(mppfStd, szField(sz,0), szField(sz,1), szField(sz,2)) {}
    ~GpibInterfaceAny() {}
  };

  class GpibInterfaceControl:public GpibInterface {
   public:
    GpibInterfaceControl(const char *sz):GpibInterface(mppfStd, "gco", szField(sz,0), szField(sz,1)) {}
    ~GpibInterfaceControl() {}
  };

  class GpibInterfaceDevice:public GpibInterface {
   public:
    GpibInterfaceDevice(const char *sz):GpibInterface(mppfStd, "gdi", szField(sz,0), szField(sz,1)) {}
    ~GpibInterfaceDevice() {}
  };

  class GpibInterfaceTest:public GpibInterface {
   public:
    GpibInterfaceTest(const char *sz):GpibInterface(mppfStd, "gti", szField(sz,0), szField(sz,1)) {}
    ~GpibInterfaceTest() {}
  };
}

#endif

