
#ifndef _GPIB_GDI_HXX_
#define _GPIB_GDI_HXX_

#include "gumi_msgs.hxx"
#include "umic/umi_driver.hxx"
#include "regs.hxx"
#include "cplRaw/sm.hxx"
#include "cplRaw/flags.hxx"
#include "igcregs.h"
#include "cards.h"
#include "gca.hxx"

namespace INdepGpib100 {
  namespace Umi=INdepUmi100;

  /*
   * NEC 7210 based interface - Device 
   */
  class GifDevice:public Umi::UmiDriver, private SmM {

   private: 
    /*
     * states
     */
    enum {
      S_POFF,
      S_BAV,	
      S_BRQ,	
      S_DCAS,
      S_DONE,	
      S_IDLE,
      S_READ,	
      S_RESP,	
      S_WAIT,	
      CNT_STATES
    };

    int cSpinWr() { return 1; }	/* default spin count for commands */
    int cSpinRd() { return 1; }	/* default spin count for writes   */

    /* variables */
    const Umi::UmiMsgUp *_pmuNext;		/* next upstream message	*/
    const Umi::UmiMsgDown *_pmdCur;		/* current output message	*/
    int _eEvent;				/* last event seen		*/
    int _eMsg;					/* last message seen		*/
    int _eStLast;				/* last state 			*/
    int eEvent() const { return _eEvent; }
    int eMsg() const { return _eMsg; }
    int eStLast() const { return _eStLast; }
    int setState(int e) { _eStLast=eStatePrev(e); return e; }

    /*
     * protocol instance
     */
    const Gpib_gdi0_Proto _pt;

    /*
     * interface configuration
     */
    GpibIfConf _gic;
    const GpibIfConf &gic() { return _gic; }

    GifCard *pca() const { return static_cast<GifCard*>(Umi::UmiDriver::pca()); }


    /* statistics */
    enum StacE {
      STAC_BYTE_IN,
      CNT_STAC
    };
    int _mpcStac[CNT_STAC];
    void incStac(StacE e, int i) { _mpcStac[e]+=i; }
    void setStac(StacE e, int i) { _mpcStac[e]=i; }


    void debug( const char *sz );
    int putUpDone( int iCtx, const Umi::UmiMsgDown *pmd , ErrE eErr, int iErr = 0 );
    bool putUpEvent(int iCtx, int eEvent);

    bool sendData(int iCtx, Umi::Pmd pmd);
    void sendDataCancel(int iCtx);
    bool recvData(int iCtx);

    Umi::UmiDriver::DownE msgInit(int iCtx, Umi::Pmd pmd );
    Umi::UmiDriver::DownE msgFini(int iCtx, Umi::Pmd pmd );

    /*
     * state machine 
     */
    bool isBavGet() { return (pca()->i_was(IGC_I1, IGC_I1_DI) || pca()->i_was(IGC_I1, IGC_I1_DET)); } 
    bool isBrq() { return (pca()->i_was(IGC_I1, IGC_I1_DO)!=0); } 
    typedef void (GifDevice::* PFhState) (int iCtx, int, Umi::Pmd pmd);		

    void hPOFF(int iCtx, int eMsg, Umi::Pmd pmd );
    void hIDLE(int iCtx, int eMsg, Umi::Pmd pmd );
    void hDCAS(int iCtx, int eMsg, Umi::Pmd pmd );
    void hREAD(int iCtx, int eMsg, Umi::Pmd pmd );
    void hWAIT(int iCtx, int eMsg, Umi::Pmd pmd );
    void  hBRQ(int iCtx, int eMsg, Umi::Pmd pmd );
    void  hBAV(int iCtx, int eMsg, Umi::Pmd pmd );
    void hRESP(int iCtx, int eMsg, Umi::Pmd pmd );
    void hDONE(int iCtx, int eMsg, Umi::Pmd pmd );
    static PFhState mphs[CNT_STATES];
    void doSm(int iCtx, int eMsg, Umi::Pmd pmd, int eEvent );
    Umi::Pmd defaultDown(Umi::Pmd pmd );

   public:

    /* debug print */
   #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<< ( std::ostream &os, const GifDevice &);
   #endif

    int cStacMax() const { return CNT_STAC; }
    int cStac(int e) const { return _mpcStac[e]; }

    char *print( char *, int );
    const Umi::UmiProto *pup(int iChan);		/* proto desc for channel */

    /* 
     * message put -- Note: timer() must have been called 
     * once before using putDown() the first time
     */ 
    int putDown(int iCtx, Umi::Pmd pmd, int iChan);

    int open(int iChan);
    void close(int iChan);

    void backenable(int iCtx, int iChan);

    /* interupt handler */
    int interrupt(int iCtx);

    /* timer handler */
    int timer(int iCtx, const Umi::Time &ti);

    GifDevice(GifCard *pca);
    virtual ~GifDevice();
  };

}
#endif

