
#ifndef _CPLRAW_INET_HXX_
#define _CPLRAW_INET_HXX_ 1

#if (!(defined (MMSYS_mingw) || defined(MMSYS_win) || defined(MMSYS_wdm)))
  #include <netinet/in.h>
  #include <arpa/inet.h>
#endif
#if ((defined (MMSYS_mingw) || defined(MMSYS_win)))
  #include <winsock.h>
#endif

#include <sys/types.h>
#include <iostream>


/*
 *	IpV4 interface address abstraction
 */
class InetIfAddr {		/* IPV4 address			*/
	in_addr _addr;		/* ret of inet_addr()		*/
	#ifndef INADDR_NONE
	  enum { INADDR_NONE=0xffffffff };
	#endif 
public:
	friend bool operator==(const InetIfAddr &iaL, const InetIfAddr &iaR) {
	    return iaL._addr.s_addr == iaR._addr.s_addr;
	}
	InetIfAddr() { _addr.s_addr=INADDR_NONE; }
	InetIfAddr(const char *sz);
	InetIfAddr(const in_addr *pin ): _addr(*pin) {}
	~InetIfAddr() {}


	bool isGood() const { return (_addr.s_addr != INADDR_NONE ); }
	friend std::ostream &operator<<(std::ostream &, const InetIfAddr &);

	/* Stevens 1990, p. 449 */
	void getIna( in_addr *pin ) const ;	/* must be good	*/
};

/*
 *	IpV4 socket abstraction 
 */
class InetSoAddr:public InetIfAddr  {	
	unsigned short _hPort;
public:
	friend bool operator==(const InetSoAddr &, const InetSoAddr &);
	InetSoAddr() { _hPort=0; }
	InetSoAddr(const char *sz, short hPort);
	InetSoAddr(const InetIfAddr &ia, short hPort);
	InetSoAddr(const sockaddr_in *psa );
	~InetSoAddr() {}

	bool isGood() const { return (InetIfAddr::isGood() && _hPort); }
	friend std::ostream &operator<<(std::ostream &, const InetSoAddr &);

	/* Stevens 1990, p. 449 */
	void getSa( sockaddr_in *psa ) const ;	/* must be good	*/
};
#endif
//#endif 
