#ifndef _NETSOCK_NET_HXX
#define _NETSOCK_NET_HXX 1

#include "cplRaw/flags.hxx"
#include "netSock/inet.hxx"
#include <iostream>
#include <fstream>


namespace INdepNet120 {

    /*
     * IMPORTANT NOTE (UNIX): 
     *
     * Because the read()/write() system calls used by
     * class netsbuf can be interrupted by signal SIGPIPE (13)
     * the signal disposition of that signal be set to SIG_IGN
     * by each application using class netsbuf 
     *
     * STRANGE Problems using g++ libstdc++ 2.10 if 
     * (netsbuf*)this != (streambuf*)this
     */
    class netsbuf : public std::streambuf, public Flags {
	int  _fd;
	int  _iTmo;			/* timeout		*/
	char *_pchBuf;
	size_t _sBuf;			/* >=2			*/

	enum {
	    F_TMO = 001,		/* timeout flag		*/
	    F_WRF = 002			/* write failed 	*/
	};

	/* put area first in buffer, then get area */
	size_t sPut() { return _sBuf/2; }
	size_t sGet() { return _sBuf-sPut(); }
	char *pchPut() { return _pchBuf; }
	char *pchGet() { return _pchBuf+sPut(); }
	char *pchPutLim() { return _pchBuf+sPut(); }
	char *pchGetLim() { return _pchBuf+_sBuf; }


	/* 
	 * virtuals
	 */
	int overflow(int);
	int underflow();
   	std::streamsize xsgetn(char *, std::streamsize); 			
   	std::streamsize xsputn(const char *, std::streamsize); 			

      public:
	int sync();
	int fdPoll() { return _fd; }		/* suitable for poll(2)	*/
	void setTmo(int iMs) { _iTmo=iMs; }
	bool wasTmo() { return (isSet(F_TMO)); }

	netsbuf(int fd);
	~netsbuf();
    };

    /*
     *	network interface functions
     */
    class CotsActive {
	InetSoAddr _isaLocal;		/* local endpoint to use */
	int _fd;
	netsbuf *setFail() { return 0; }
      public:
	netsbuf *connect(const InetSoAddr &isa);
	CotsActive();
	~CotsActive();
    };

    class CotsPassive:public Flags {
	int _fd;
	int _cBackLog;

	enum {
	    F_BAD = 001
	};

	CotsPassive& setFail() { setFlag( F_BAD ); return *this; }
	netsbuf *setFailBuf() { setFlag( F_BAD ); return 0; }
      public:
	bool isGood() { return isClear(F_BAD); }
	int fdPoll() { return _fd; }		/* suitable for poll(2)	*/

	CotsPassive& open(const InetSoAddr &isa);
	netsbuf* listen(InetSoAddr *pIsa);
	CotsPassive();
	~CotsPassive();
    };
}

#ifdef NEVER
struct nd_addrlist;

class netsbuf;
class NetTrans {
	struct netconfig *_pnc;			/* transport description			*/
	nd_addrlist *_pal;			/* list of addresses for selected service	*/
	int _fdOpen;				/* end point opened (active or passive)		*/

	netbuf *nbcopy(netbuf* pnbD, netbuf *pnbS);
	int nbcmp(netbuf* pnbL, netbuf *pnbR);

	void log(Erf&);

public:
	/*
	 *	openPassive()
	 *
	 *	passive open (server)
	 *	
	 *	opens an listening endpoint using queue length cConn
	 *	nonzero on error.
	 */	
	int openPassive(int cConn);

	/*
	 *	openActive()
	 *
	 *	active open of endpoint. netsbuf may be used even if this is
	 *	destructed.
	 *
	 *	zero on error.
	 */
	netsbuf* openActive( const InetSoAddr &isa, const char *szNsId = 0 );

	/*
	 *	fd of opened connection
	 */
	int fdPollListen();

	/*
	 *	accept()
	 *
	 *	wait for and accept a new connection 
	 *	after a passive open.
	 *
	 *	The descriptor of the new connection is returned via *pfd or *ppsb.
	 *	The universal address of the new connection is returned 
	 *	via szAddr (if nonzero).
	 *
	 *	Any remote endpoint is accepted.
	 *
	 *	returns nonzero on error.
	 */
	netsbuf* listen( Erf &erf,  char *szAddr, size_t sAddr );

	/*
	 *	close ??
	 */

	NetTransServer();
	~NetTrans();
};
#endif

#endif
