#ifndef _UMI_UMI_USER_HXX
#define _UMI_UMI_USER_HXX

/*
 * Universal Message Interface - User C++ part
 */
#include "umic/umi.hxx"
#include "cplRaw/failable.hxx"

#ifndef EOF 
  #define EOF (-1)
#endif


namespace INdepUmi100 {



  /*
   * Universal Message Interface
   *
   * 'Failure' in the context of a Umi always means 'transportation' 
   * never 'endpoint' failure.
   */
  class Umi:public Failable {
    const UmiProto *_puf;
   protected:
    //char _mz[80];


    //UmiControl *setFailed() { _eStat=-1; return 0; }
    //UmiControl *setFailed(int e) { _eStat=e; return 0; }
    //int eStat() const { return _eStat; }
    const UmiProto *puf() { return _puf; }


   public:
    /*
     * transport failures
     */
    enum {
     OK,
     E_NOBUF
    };

    /*
     * send message
     */
    virtual Failable& putMsg(UmiControl*puc, const UmiData*pud) = 0;

    /*
     * get message
     */
    virtual Failable& getMsg(UmiControlBuf *puc, UmiData *pud) = 0;
    virtual Failable& xchMsg(UmiControlBuf *puc, UmiData *pud);

    /* flow control prevent deadlock */
    virtual bool canPut() ;


    /* data max per message */
    size_t sDataMax() { return 64; }		

    Umi(const UmiProto *puf);
    virtual ~Umi();
  };



  class Factory {
   public:
    /*
     * create channel to driver specified by protocol and channel number
     * (thread-safe)
     */
    virtual Failable createUmi(Umi**ppum, const UmiProto *, int iChan) const =0;

    Factory() {} 
    virtual ~Factory() {}
  };

  /* type of factory maker functions */
  typedef Factory* (*PFF) (const char *szMode, const char *szArg);

  /*
   * implementation dependent list of Factories
   * (thread-safe)
   */
  class Factories {
   public:

    /*
     * creates a new factory bound to a driver 
     * with the following properties:
     *
     * szMode = access domain (l)ocal, (k)ernel, (n)et
     * szArg = location interpreted by domain
     *
     * The factory created is able to connect to all
     * channels of the driver.
     * 
     * The factory must be deleted by the caller.
     */
    virtual Factory* puf(const char *szMode, const char *szArg) const = 0;

    Factories() {}
    virtual ~Factories() {}
  };

}
#endif


