#ifndef _UMI_H
#define _UMI_H

/*
 * Universal Message Interface - C part
 */
#include <stddef.h>

/* 
 * ioctls
 */
#define GIOC_PROCHK 0	/* check protocol			*/
#define GIOC_PUTMSG 1	/* put a message to the kernel		*/
#define GIOC_GETMSG 2	/* get a message from the kernel	*/
#define GIOC_EXCMSG 3	/* exchange messages with the kernel	*/


#ifdef __cplusplus
extern "C" {
#endif

  #define SZ_DATA_MAX 512
  #define SZ_CTRL_MAX 128	/* including header	*/

  
  /*
   * incore rep of the message data part
   * 
   */
  struct UmiDataStruct {
    /* buffer */
    char *_pchLimBuf;	/* last byte +1 of buffer	 */
    char *_pchLimDat;	/* last byte +1 of valid data	 */
    char *_pchBase;	/* first byte of allocation	 */
  };

  /*
   * message types (for flow control)
   */
  #define MT_RQ	  1	/* request (expecting response)  */
  #define MT_RP	  2	/* response		         */
  #define MT_IN   3	/* indication (no req/resp)	 */

  /*
   * any message header
   */
  struct UmiControlStruct {


    /*
     * size and type of this
     */
    size_t    _size;	 	/* intra-machine copy  	*/
    unsigned  _eType;	 	/* message type		*/

    /*
     * pointers to data part elsewhere, 
     * required for the user/kernel boundary only
     */
    struct UmiDataStruct _ud;

    /* more control follows here 	*/
    /* ....		 		*/

  };

  /* 
   * result codes for putDown() of the driver
   */
  #define MSG_DONE 0	/* task done			*/
  #define MSG_WAIT 1	/* task remains		   	*/
  #define MSG_BUSY 2	/* not able to create task 	*/

  struct timespec;





#ifdef __cplusplus
}
#endif

#endif
