#ifndef _UMI_C_H_
#define _UMI_C_H_

#include <stddef.h>
#include "umi.h"


  struct UmiIter {
     char _mpch[8*8];
  };

  struct UmiMemInfo {
    size_t _s;
    void * _p;
  };

  struct UmiCard ;
  struct UmiDriver ;
  struct UmiChannel ;
  struct UmiMsgDownBuf ;
  struct UmiMsgUpBuf ;
  struct CardType ;

  struct UmiEnvC {
    void (*_pFtiNextTick)(struct timespec *);
    void (*_pFsigGet)(int, void*);
    void (*_pFsigPut)(int, void*);
  };


  /*
   * module specifics methods
   */
  struct UmiMod {
    int _cChan;			/* channel per card		*/
    int _tps;			/* timer ticks per second	*/
    void *(*_pFnew_UmiCard)(struct UmiMod*, const struct UmiEnvC*, struct UmiMemInfo*, int, const struct CardType *, long lAddr);
    int (*_pFgetMem_UmiCard)(struct UmiMod*, struct UmiMemInfo*, int);
  };

  extern int UmiCard_timer( struct UmiCard*, int, const struct timespec *);
  extern int UmiCard_interrupt( struct UmiCard*, int );

  /*
   * generic methods
   */
  extern void UmiCard_startIterDrivers( struct UmiCard*, struct UmiIter *);
  extern struct UmiDriver * UmiCard_iterateDrivers( struct UmiCard*, struct UmiIter *);
  extern const char * UmiDriver_szName( struct UmiDriver *);
  extern char * UmiDriver_print( struct UmiDriver *, char *, int );

  extern void UmiDriver_startIterChannels( struct UmiDriver*, struct UmiIter *);
  extern struct UmiChannel* UmiDriver_iterateChannels( struct UmiDriver*, struct UmiIter *);
  extern int UmiChannel_num( struct UmiChannel *);
  extern int UmiChannel_open( struct UmiChannel *);
  extern const char* UmiChannel_szProto( struct UmiChannel *);
  extern void UmiChannel_close( struct UmiChannel *);
  extern void UmiChannel_setHandle( struct UmiChannel *, void *p);

  /*
   * channel upstream 
   */
  extern int UmiChannel_canget( struct UmiChannel *);
  extern struct UmiMsgUpBuf *UmiChannel_getUpBuf( struct UmiChannel *);			/* next to get		*/
  extern void UmiChannel_releaseUpBuf( struct UmiChannel *, int iCtx, struct UmiMsgUpBuf *);	/* last got		*/

  /*
   * channel downstream 
   */
  extern int UmiChannel_canput( struct UmiChannel *);
  extern struct UmiMsgDownBuf *UmiChannel_allocDownBuf( struct UmiChannel *);	/* next to put		*/
  extern void UmiChannel_putDownBuf( struct UmiChannel *, int iCtx, struct UmiMsgDownBuf *); /* last got		*/

  extern size_t UmiMsgUpBuf_sizeControl(struct UmiMsgUpBuf *);
  extern struct UmiControlStruct * UmiMsgUpBuf_pControl(struct UmiMsgUpBuf *);
  extern size_t UmiMsgUpBuf_sizeData(struct UmiMsgUpBuf *);
  extern char * UmiMsgUpBuf_pData(struct UmiMsgUpBuf *);

  extern size_t UmiMsgDownBuf_sizeControl(struct UmiMsgDownBuf *);
  extern size_t UmiMsgDownBuf_sizeDataMax(struct UmiMsgDownBuf *);
  extern void UmiMsgDownBuf_setDataLim(struct UmiMsgDownBuf *, size_t);
  extern struct UmiControlStruct * UmiMsgDownBuf_pControl(struct UmiMsgDownBuf *);

  extern char * UmiMsgDownBuf_pData(struct UmiMsgDownBuf *);

  #define UmiMod_cChan(pum) (pum->_cChan)
  #define UmiCard_new(pum, pue, a, b, c, d) ((*pum->_pFnew_UmiCard)((pum), (pue), (a), (b), (c), (d)))
  #define UmiCard_getMem(pum, a, b) ((*pum->_pFgetMem_UmiCard)(pum, (a), (b)))

  #define MAX_UMI_MEM 32

#endif
