#include "port.h"
/* /home/INdep/src/shelf/cplRaw.100/sm.hxx */


#ifndef _CPLRAW_SM_HXX
#define _CPLRAW_SM_HXX 1

/*
 *	sm.hxx
 *	ines foundation state machine class
 *
 *	970827	ws	created
 */



/*
 *	minimal state machine
 */
class SmM {
	int _e;			/* current state	*/
public:
	int eState() const { return _e; }
	bool isState(int e) { return _e==e; }
	bool isState(int e1, int e2) { return (_e==e1 || _e == e2); }
	int setState(int e) { return _e=e; }
	int eStatePrev( int e ) {
		int ePrev=_e;
		_e=e;
		return ePrev;
	}

	/*
	 *	set state to eNew,
	 *	return true if old state was ePrev
	 */
	int wasState(int eNew, int ePrev);

	SmM(int e=0);	
	~SmM();
};



#endif 
/* /home/INdep/src/shelf/cplRaw.100/list.hxx */

#ifndef _BASE_LIST_HXX_
#define _BASE_LIST_HXX_ 1

/*
 *	Copyright (C)  by
 *	ines GmbH 1991
 */



#ifndef _BASE_LIST_HXX
#define	_BASE_LIST_HXX 1

typedef unsigned uint;

class ChainRaw;

/*
 *	LIST NOMENCLATURE

	Link:	+---+	+---+   +---+
		  n  ->   n  ->   n\
		 \p  <-   p  <-   p
		+---+	+---+   +---+
	Chain:	^		^
		head		tail

	[ direction: head to tail ]

	put(el, nel)		append nel to el 
	put(0, nel)		insert nel @head
	put(tail(), nel) 	append to list [push]
	
	remove(el)		remove elem from list
	remove(head())		remove head element [pull]
	remove(tail())		remove tail element [pop]
 */

/*
 *	a Link 
 *
 *	if a Link can be on different chains, and this is significant.
 *	deletion of a Link will remove it automatic from it's chain.
 */

class Link {
	friend class ChainRaw;
	friend class Chain;

	ChainRaw *_chain;
	Link *_next;
	Link *_prev;
public:
	Link();				// construct isolated (!) Link
	Link(ChainRaw *);		// construct a Link, append it to chain
	virtual ~Link();		// remove also from chain
	ChainRaw* chain() const { return _chain; }
	int isIsol() { return (_chain==0); }
					// get chain or null
	Link *next() { return _next; }
	Link *prev() { return _prev; }
};

struct ChainIterator {
	Link*	_pliCur;	/* current during iteration */
	int	_fTH;		/* iterate for tail to head if set */
};
#define ChainIter ChainIterator 

class ChainRaw {
	Link *_head;
	Link *_tail;
	uint  _c;

public:
	ChainRaw(); 			// construct
	ChainRaw(ChainRaw*);		// imports source chain, doesn't duplicate links!
	~ChainRaw();			// doesn't delete links

	/*
	 *	start iteration
	 *	you can iterate from head to tail or reversed
	 */
	void start_iter(ChainIterator *pit)	/* obsolete */
		const { pit->_pliCur = _head; pit->_fTH=0; }
	void strtIter(ChainIterator *pit) const   { start_iter(pit); }
	void strtIterHT(ChainIterator *pit) const { start_iter(pit); }
	void strtIterTH(ChainIterator *pit) const { pit->_pliCur=_tail; pit->_fTH=1; }

	/*
	 *	iterate
	 *	returns 0 if iteration done. 
	 *	Direction is given by startIter??.
	 *	save during iteration:
	 *		Deletion/removal of the current node
	 *	Usage:
	 *
	 *	strtIter(&it)
	 *	while (pl=iterate(&it)) {
	 *		If (...) delete(pl);
	 *	}
	 */
	Link* iterate(ChainIterator*) const;	
	Link* iter(ChainIterator* pit)	const { return iterate(pit); }

	/*
	 *	peek()
	 *	returns the next link to be returned by iter() without
	 *	moving forward/backward. 
	 *	Nodes appended( during head/tail iter ) to peek() 
	 *	are included in the iteration. 
	 *	Deletion/removal of peek() is not allowed. 
	 *
	 *	Usage (a): Append during iteration
	 *
	 *	strtIter(&it)
	 *	while (pl=peek(&it)) {
	 *		If (...) append(pl, new Link());
	 *		(void)iter();
	 *	}
	 *
	 *	Usage (b): Test for list end.
	 *
	 *	strtIter(&it)
	 *	while (pl=iter(&it)) {
	 *		cout << (*pl);
	 *		if (peek()) cout << ','; // not at end
	 *		(void)iter();
	 *	}
 	 *	
	 */
	Link *peek(ChainIterator *) const ;

	Link* head() const;		// first elem or null
	Link* tail() const;
	Link* put(Link*l, Link*nel);	// put nel behind 'l', ret nel or 0
	Link* append(Link*l, Link*nel) { return put(l, nel); }
	Link* insert(Link*l, Link*nel) { return put(l ? l->prev() : 0, nel); }

	Link* remove(Link*);		// remove Link from list,fatal if not
	Link* push(Link *l)		// append to chain, fatal if on oth ch
		{ return put(_tail, l); }
	Link  *pop()			// remove last elem
		{ return remove(_tail); }
	Link  *pull()			// remove first elem
		{ return remove(_head); }
	uint count() const { return _c; }	// count links on chain
	uint c() const { return count(); }	// new name 

	/*  read chain to array */
	int get(Link **, int c);
	Link *get(int iPos) const;
	int pos(const Link *pln) const;

	/*
	 *	import()
	 *	remove from other chain, append to this
	 */
	Link* import(Link *pln) {	
		if (pln->chain()) pln->chain()->remove(pln);
		this->push(pln);
		return pln;
	}
	void import(ChainRaw *pcn);	/* return this	*/
};


/*
 *	CHAINITER macro
 *	simplifies coding of chain iterations
 *	usage:
 *	
 *	class X:public Link { int _y; };
 *	Chain ch;
 *	CHAINITER( ch, X*, pX) {
 *		 if (pX->_y) ...
 *	}
 */
#define CHAINITER( chname, linkcast, linkvar ) \
	ChainIterator cit;\
	linkcast linkvar;\
	chname.strtIter(&cit);\
	while(linkvar=(linkcast)chname.iter(&cit))
#endif

/*
 *	Lightweight Lists
 *
 *	- no explicit chain object
 *	- no recursive deletion
 */
class LiLink {
	LiLink *_pllNext;
	LiLink *_pllPrev;
public:
	LiLink();
	~LiLink();
	
	LiLink *pllNext() { return _pllNext; }
	LiLink *pllPrev() { return _pllPrev; }

	int isHead() { return _pllPrev == 0; }
	int isTail() { return _pllNext == 0; }
	int isIsol() { return ((!_pllNext) && (!_pllPrev)); }


	LiLink *append(LiLink *pllNew);		/* after this	*/
	LiLink *insert(LiLink *pllNew);		/* before this	*/
	LiLink *remove();
};

/*
 *	Lightweight Chain - no recursive deletion
 */
class LiChain {
	LiLink *_pllHead;
	LiLink *_pllTail;
public:
	LiChain();
	~LiChain();			/* no deletion */
	
	LiLink *appendTail(LiLink *pll);/* append to tail */
	LiLink *removeHead();		/* link or 0	*/
	LiLink *remove(LiLink *pll);		/* link or 0	*/
};

/*
 *	Lightweight List with payload
 */
class LiLiPl:public LiLink {
	void *_pv;
public:
	LiLiPl(void *p);		/* use LiLink if mounting not required */
	~LiLiPl();

	void *pv() { return _pv ; }
	void *pvNext() { return pllNext() ? ((LiLiPl*)pllNext())->_pv : 0; }
	void *pvPrev() { return pllPrev() ? ((LiLiPl*)pllPrev())->_pv : 0; }
};

#endif
/* /home/INdep/src/shelf/cplRaw.100/failable.hxx */
#ifndef _CPLRAW_FAILABLE_HXX_
#define _CPLRAW_FAILABLE_HXX_

#ifndef _MM_NO_IOSTREAM
#endif 

  /*
   * 
   */
  class Failable {
    enum StatE {
      E_OK,
      E_FAIL = 001,
      E_BAD =  002 ,
      E_NEXT = 004
    }  _eStat;

    int _iErr;
    const char *_szErr;		/* must be static	*/

   protected:
    Failable& Failable::setStat(StatE eStat, const char *sz, int i);
    Failable& Failable::setStat(const Failable &fa);


    #ifndef _MM_NO_IOSTREAM
      friend std::ostream &operator<<(std::ostream &, const Failable &);
    #endif
   public:
    bool isGood() const { return _eStat == 0; }				/* fine 		*/
    int print(char *sz, unsigned size) const;
    int iErr() { return _iErr; }
    const char *szErr() { return _szErr; }

    /* 
     * last op failed, but object is uncorrupted
     */
    bool isFail() const { return (_eStat & (E_FAIL|E_BAD))!=0; }

    /* 
     * last op failed, object is corrupted
     */
    bool isBad() const { return (_eStat & E_BAD) != 0; }

    bool operator!() const { return isFail(); }
    operator const void * () const { return isFail() ? 0 : this ; }

    /* to be implemented */
    Failable &setFailed(const char*sz, int i, const Failable *) { return setStat(E_FAIL, sz, i); }
    Failable &setBad(const char*sz, int i, const Failable *) { return setStat(E_BAD, sz, i); }

    Failable &setFailed(const char*sz, int i=0) { return setStat(E_FAIL, sz, i); }
    Failable &setBad(const char*sz, int i=0) { return setStat(E_BAD, sz, i); }

    Failable();
    ~Failable(); 
  };

#endif
/* /home/INdep/src/shelf/cplRaw.100/stats.hxx */

#ifndef _BASE_STATS_HXX_
#define _BASE_STATS_HXX_ 1

#ifndef _MM_NO_IOSTREAM
#endif 

/*
 * statistics recorder
 */
class Counter {
	bool _fOverflow;
	unsigned _c;	/* number of bytes	*/
	unsigned _n;	/* number of calls	*/

	bool isOverflow() const { return _fOverflow; }
	void setOverflow() { _fOverflow=true; }
public:
	void sample(unsigned n);
	int c() const { return _c; }
      #ifndef _MM_NO_IOSTREAM
	friend std::ostream &operator<<(std::ostream&, const Counter &);
      #endif 

	Counter();
	~Counter();
};

#endif
/* /home/INdep/src/shelf/cplRaw.100/dbg.hxx */

#ifndef _BASE_DBG_HXX
#define _BASE_DBG_HXX 1


/*
 *	dbg.hxx
 *	DEBUG utilities
 */

#ifndef _MM_NO_IOSTREAM

extern "C" { int addone(int i); }

extern void debug_showbuf(std::ostream &os, const char *pch, int c);  /* printable & hex */
extern void debug_showhex(std::ostream &os, const char *pch, int c, int chDel='.');  /* hex only */
inline void debug_showhex(std::ostream &os, const void *p, int c, int chDel='.') {
    debug_showhex(os, static_cast<const char *>(p), c, chDel);  /* hex only */
}
inline void debug_showbuf(std::ostream &os, const void *p, int c) {
    debug_showbuf(os, static_cast<const char *>(p), c);
}

#endif
#endif
/* /home/INdep/src/gpib/umi.112/umi.h */
#ifndef _UMI_H
#define _UMI_H

/*
 * Universal Message Interface - C part
 */

/* 
 * ioctls
 */
#define GIOC_PUTMSG 1	/* put a message to the kernel		*/
#define GIOC_GETMSG 2	/* get a message from the kernel	*/
#define GIOC_EXCMSG 3	/* exchange messages with the kernel	*/


#ifdef __cplusplus
extern "C" {
#endif

  #define SZ_DATA_MAX 512
  #define SZ_CTRL_MAX 128	/* including header	*/

  
  /*
   * incore rep of the message data part
   * 
   */
  struct UmiDataStruct {
    /* buffer */
    char *_pchLimBuf;	/* last byte +1 of buffer	 */
    char *_pchLimDat;	/* last byte +1 of valid data	 */
    char *_pchBase;	/* first byte of allocation	 */
  };

  /*
   * message types (for flow control)
   */
  #define MT_RQ	  1	/* request (expecting response)  */
  #define MT_RP	  2	/* response		         */
  #define MT_IN   3	/* indication (no req/resp)	 */

  /*
   * any message header
   */
  struct UmiControlStruct {


    /*
     * size and type of this
     */
    size_t    _size;	 	/* intra-machine copy  	*/
    unsigned  _eType;	 	/* message type		*/

    /*
     * pointers to data part elsewhere, 
     * required for the user/kernel boundary only
     */
    struct UmiDataStruct _ud;

    /* more control follows here 	*/
    /* ....		 		*/

  };

  /* 
   * result codes for putDown() of the driver
   */
  #define MSG_DONE 0	/* task done			*/
  #define MSG_WAIT 1	/* task remains		   	*/
  #define MSG_BUSY 2	/* not able to create task 	*/

  struct timespec;





#ifdef __cplusplus
}
#endif

#endif
/* /home/INdep/src/gpib/umi.112/umi.hxx */
#ifndef _UMI_UMI_HXX
#define _UMI_UMI_HXX

/*
 * Universal Message Interface - Generic C++ part
 */
#if (defined(MMSYS_linux) || defined(MMSYS_lkern))
#else
  struct timespec {
    unsigned tv_sec;
    unsigned tv_nsec;
  };
#endif

#ifndef _MM_NO_IOSTREAM
#endif

#ifndef EOF 
  #define EOF (-1)
#endif



class xostream;
class xistream;

namespace INdepUmi100 {

  class UmiControl:public UmiControlStruct {
   public:
    bool isType(unsigned e) { return _eType == e; }
    int eType() const { return _eType; }
    size_t size() const { return _size; }

    //size_t sizeData() const { return _pchLim-_pchBase; }
    friend xostream &xmarshal(xostream&, const UmiControl &) ;
    friend xistream &xunmarsh(xistream &, UmiControl &);

    void iLength(int) {}
  #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<<(std::ostream &, const UmiControl &);
  #endif
    friend xistream &operator>>(xistream &, UmiControl &);

    UmiControl() { _eType=0, _size=0, _ud._pchBase=_ud._pchLimDat=_ud._pchLimBuf=0; }
    UmiControl(int eType, size_t s)  { _eType=eType, _size=s, _ud._pchBase=_ud._pchLimDat=_ud._pchLimBuf=0; }
    ~UmiControl() { }
  };

  /*
   * user level data descriptor, used to pass buffer information
   * the UM interface
   */
  class UmiData:public UmiDataStruct {
   public:
    //size_t sizeMax() const { return SZ_DATA_MAX ; }
    char *pchBase() const { return _pchBase; }
    char *pchLimBuf() const { return _pchLimBuf; }
    char *pchLimDat() const { return _pchLimDat; }
    void setSize(size_t s) { _pchLimDat = _pchBase + s; }
    size_t cData() const { return _pchLimDat-_pchBase; }
    size_t cMax() const { return _pchLimBuf-_pchBase; }

    //UmiData& operator=(const UmiDataStruct &); /* copy in data  */

    UmiData() {
      _pchLimDat=_pchLimBuf=_pchBase=0;
    }
    UmiData(char *pch, size_t s) {
      _pchBase=pch;
      _pchLimDat=_pchLimBuf=pch+s;
    }
    ~UmiData() { }
  };

  class UmiControlBuf:public UmiControl {
    char _mpch[SZ_CTRL_MAX];
  public:
    UmiControlBuf& operator=(const UmiControl &); /* copy in any UmiControl */
    UmiControlBuf();
    ~UmiControlBuf();
  };



  class UmiDataBuf:public UmiData {
    char _mpch[ SZ_DATA_MAX ];
   public:
    UmiDataBuf():UmiData(_mpch, SZ_DATA_MAX) {}
    ~UmiDataBuf() {}
  };

  /*
   * timesp.hxx
   * Posix 1b timespec c++ wrapper class
   */
  class Time:public timespec {
   public:
    friend Time operator+( const Time &tiL, const Time &tiR );
    friend bool operator>=( const Time &tiL, const Time &tiR );
    friend xistream &xunmarsh( xistream &, Time & );
    friend xostream &xmarshal( xostream &, const Time &) ;
    #ifndef _MM_NO_IOSTREAM
      friend std::ostream &operator<<(std::ostream &, const Time &);
    #endif

    Time() { tv_sec=0; tv_nsec=0; }   /* default time value is 0.0 seconds */
    Time( const Time &ti ) { tv_sec=ti.tv_sec; tv_nsec=ti.tv_nsec; }
    Time( const struct timespec &ti ) { tv_sec=ti.tv_sec; tv_nsec=ti.tv_nsec; }
    Time( unsigned sec, unsigned nsec=0 ) { tv_sec=sec; tv_nsec=nsec; }
    ~Time() {}
  };

  /*
   * UmiGetBuf, UmiPutBuf -- universal put/get buffers
   */
  class UmiGetBuf {
    const char * const _pchBase;	/* first byte of allocation	 */
    const char * const _pchLimBuf;	/* last byte +1 of buffer	 */

    const char *_pchLimDat;		/* last byte +1 of valid data	 */
    const char *_pch;			/* next byte to get		 */

    int zapeof(const char *pch) { return *reinterpret_cast<const unsigned char *>(pch); }

   public:
    /* get */
    int cDone() { return _pch-_pchBase; }	/* chars done	   */
    int sgetc() { return (_pch >= _pchLimDat) ? EOF : zapeof(_pch); }
    int sbumpc() { return (_pch >= _pchLimDat) ? EOF : zapeof(_pch++); }
    int sbumpc_checked() { return zapeof(_pch++); }  /* optimized version */

    const char *pchBase() const { return _pchBase; }
    int cMax() const { return _pchLimBuf-_pchBase; }		/* chars max */
    void setCount(size_t c) { _pchLimDat=(_pch=_pchBase)+c; }

    UmiGetBuf(char *pchBase, size_t s): _pchBase(pchBase), _pchLimBuf(pchBase+s), _pchLimDat(pchBase), _pch(pchBase) { }
    ~UmiGetBuf() { }

  };

  class UmiPutBuf  {
    char * const _pchBase;	/* first byte of allocation	 */
    char * const _pchLimBuf;	/* last byte +1 of buffer	 */

    char *_pchLimDat;		/* last byte +1 of valid data	 */
    char *_pch;			/* next byte to put		 */
   public:
    /* put */
    bool canput() { return _pch < _pchLimDat; }
    bool canputnext() { return (_pch+1) < _pchLimDat; }
    void sputc( int ch ) { *(_pch++)=(unsigned char)ch; }
    size_t cDone() const { return _pch-_pchBase; }		/* chars done	   */
    char *pchBase() const { return _pchBase; }
    int cRem() { return _pchLimDat-_pch; }		/* chars remaining */

    /*
     * set the limit of bytes the driver accepts before
     * returning !canput(). This limit is independent of
     * the buffer size.
     */
    void setLimit(size_t s) { _pch=_pchBase, _pchLimDat=_pchBase+s; }

    UmiPutBuf(char *pchBase, size_t s): _pchBase(pchBase), _pchLimBuf(pchBase+s), _pchLimDat(pchBase), _pch(pchBase) { }
    ~UmiPutBuf() { }
  };

  /* protocol information */
  class UmiProto {
    const char *_szProto;
   public:

   #ifndef _MM_NO_IOSTREAM
    virtual xistream& unmarsh(xistream &, const UmiControl &, UmiControl *) const = 0;
    virtual xostream& marshal(xostream &, const UmiControl *) const = 0;
   #endif

    const char *szProto() const { return _szProto; }
    UmiProto(const char *szProto):_szProto(szProto) {}
    virtual ~UmiProto() {}
  };
}

#endif


/* /home/INdep/src/gpib/umi.112/umi_driver.hxx */
#ifndef _UMI_UMI_DRIVER_HXX
#define _UMI_UMI_DRIVER_HXX

/*
 * Universal Message Interface - Driver C++ part
 */



namespace INdepUmi100 {

  /*
   * kernel message reps
   */
  class UmiMsgDown:public Link {
    UmiControl *_puc;
    UmiGetBuf *_pug;
   public:
    const UmiControl *puc() const { return _puc; }
    UmiGetBuf *pug() const { return _pug; }
    void init(UmiControl *puc, UmiGetBuf *pug) { _puc=puc; _pug=pug; }
    UmiMsgDown():_puc(0), _pug(0) {}
    ~UmiMsgDown() {}
  };

  class UmiMsgUp:public Link {
    UmiControl *_puc;
    UmiPutBuf *_pup;
   public:
    UmiControl *puc() const { return _puc; }
    UmiPutBuf *pup() const { return _pup; }

    void init(UmiControl *puc, UmiPutBuf *pup) { _puc=puc; _pup=pup; }
    UmiMsgUp():_puc(0), _pup(0) {}
    ~UmiMsgUp() {}
  };

  typedef const UmiMsgDown *Pmd;
  typedef const UmiMsgUp *Pmu;


  /*
   * self-contained message copies
   */
  class UmiMsgDownBuf:public UmiMsgDown, public UmiGetBuf, public UmiControlBuf { 
    char _mpchData[ SZ_DATA_MAX ];
   public:
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream &operator<<(std::ostream &os, UmiMsgDownBuf &ud);
    #endif

    /*
     * copy in user messages
     */
    UmiControl& operator=(const UmiControl &uc) { return UmiControlBuf::operator=(uc); }

    UmiMsgDownBuf() ;
    ~UmiMsgDownBuf() ;
  };

  class UmiMsgUpBuf:public UmiMsgUp, public UmiPutBuf, public UmiControlBuf {
    char _mpchData[ SZ_DATA_MAX ];
   public:
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream &operator<<(std::ostream &os, UmiMsgUpBuf &ud);
    #endif

    UmiMsgUpBuf();
    ~UmiMsgUpBuf();
  };

  /*
   * user/driver copy
   */
  void copy(UmiData *, const UmiMsgUpBuf*);
  void copy(UmiMsgDownBuf*, const UmiData*);

  /*
   * upper message handler interface definition
   */

  class UmiEnv {
   public:
    virtual Time tiNext( const Time &ti )=0;		
    virtual void sigGet( int, void * );		
    virtual void sigPut( int, void * );		
    UmiEnv() {}
    virtual ~UmiEnv() {}
  };

  class UmiDriver;
  class UmiChannel : public Link {
    /* driver */
    UmiDriver*  _pud;
    UmiEnv*	_pue;

    /* channel number, identifies channel @ driver */
    int	        _iChan;		/* chan @ _pud			*/
    void*	_phEnv;		/* handle at _pue		*/

    /*
     * message pools
     */
    ChainRaw _cnUmuPool;
    ChainRaw _cnUmdPool;

    /*
     * downstream messages
     */
    ChainRaw _cnUmdQueue; 			/* messages queued for free channels 	*/
    ChainRaw _cnUmdDriver;			/* per channel message (single)		*/
    ChainRaw _cnUmdUser;			/* in process by user			*/

    /* 
     * upstream messages 
     */
    ChainRaw _cnUmuQueue;			/* sent upstream 			*/
    ChainRaw _cnUmuDriver;			/* in process by driver			*/
    ChainRaw _cnUmuUser;			/* in process by user			*/

    ChainRaw *pcnUmuPool() { return &_cnUmuPool; }
    ChainRaw *pcnUmdPool() { return &_cnUmdPool; }
    ChainRaw *pcnUmdUser() { return &_cnUmdUser; }	

    ChainRaw *pcnUmuDriver() { return &_cnUmuDriver; }	/* allocated messages	*/
    ChainRaw *pcnUmuQueue() { return &_cnUmuQueue; }	/* msgs sent upstream   */
    ChainRaw *pcnUmuUser() { return &_cnUmuUser; }	

    ChainRaw *pcnUmdQueue() { return &_cnUmdQueue; }
    ChainRaw *pcnUmdDriver() { return &_cnUmdDriver; }
   public:
    /* init buffers  */
    void initPool(UmiMsgDownBuf *pUmd) { _cnUmdPool.push(pUmd); }
    void initPool(UmiMsgUpBuf *pUmu) { _cnUmuPool.push(pUmu); }

    /* move all back to the pools and call finiPool()	*/
    void finiQueues();			
    void finiPools();			
    virtual void finiPool(Link *);

    #ifndef _MM_NO_IOSTREAM
     std::ostream &printQueues(std::ostream &os) const ;
    #endif 

    UmiDriver *pud() { return _pud; }
    UmiEnv *pue() { return _pue; }
    int iChan() { return _iChan; }

    /*
     * synchronous open/close
     * only one open()/close() per use. close and reopen
     * the channel for reset.
     */
    virtual int open();
    virtual void close();

    /*
     * pollIn, Out
     */
    int pollIn()  { return pcnUmuQueue()->c(); } 
    UmiMsgUpBuf*getUpBuf();		/* get from up queue, iff pollIn()	*/
    void releaseUpBuf(int iCtx, UmiMsgUpBuf *);	/* most recent got from getUpBuf()	*/

    int pollOut() { return pcnUmdPool()->c(); } 
    UmiMsgDownBuf* allocDownBuf();
    void putDownBuf(int iCtx, UmiMsgDownBuf*);	/* put to driver		*/

    /*
     * send a message upstream, message and buffer must have been obtained
     * by allocUp and are released be this call
     */
    virtual void putUp(int iCtx, Pmu );

    /*
     * allocate an upstream message
     */
    virtual Pmu allocUp(size_t sData);

    /*
     * allocate an upstream message. If the queue is
     * blocked, remove the last message putUp() and reallocate it.
     */
    virtual Pmu allocUpForced(size_t sData);

    /*
     * allocate an upstream message. Declares that the control
     * and data buffers of pmd are no more used and may be
     * also used for the upstream message. 
     */
    virtual Pmu allocUp(size_t sData, Pmd pmd);


    /*
     * release an downstream message, ANY downstream message must
     * be released this way. Clears channel. 
     * Driver::putDown MAY BE CALLED DURING releaseDown() FOR THE CHANNEL.
     */
    virtual void releaseDown(int iCtx, Pmd);

    /* 
     * next timer tick >= now + ti (depends on system clock granularity)
     */
    Time tiNext( const Time &ti ) { return pue()->tiNext(ti); }


    const UmiProto *pup(); 

    void setHandle(void *p) { _phEnv=p; }
    void setDriver(UmiDriver *pud) { _pud=pud; }
    void setChan(int i) { _iChan=i; }
    UmiChannel(UmiDriver *pud, int iChan, UmiEnv *pue);
    virtual ~UmiChannel();
  };

  /*
   * timer interface def
   */
  class UmiCard;
  /*
   * lower message handler (driver) base class 
   */
  class UmiDriver : public Link {
   public:
    typedef int DownE;
   private:
    const char *_szName;
    UmiChannel *_pcn;
    UmiCard *_pca;
    ChainRaw _cnuc;			/* channel list		*/

   protected:
    Time tiNext( const Time &ti ) { return pcn(0)->tiNext(ti); }

   public:
    UmiChannel *pcn(int i) { return static_cast<UmiChannel*>(_cnuc.get(i)); }
    int ccn() { return _cnuc.c(); }

    void startIterChannels( ChainIterator *pit) { _cnuc.strtIter(pit); }
    UmiChannel *iterChannels( ChainIterator *pit) { return static_cast<UmiChannel*>(_cnuc.iter(pit)); }

    UmiCard *pca() const { return _pca; }
    const char *szName() { return _szName ; }		/* driver name, qualified by card's name */



    /* 
     * downstream message put - buffer may be reused for
     * upstream message transfer, see pha()->allocUp().
     * The message channel is locked until pha()->releaseDown() is called.
     */ 
    virtual DownE  putDown(int iCtx, Pmd pmd, int iChan)=0;

    /*
     * per channel protocol descriptor
     */
    virtual const UmiProto *pup(int iChan)=0;

    /*
     * reset any activity related to iChan (open/close processing)
     */
    virtual int open(int iChan);
    virtual void close(int iChan);
    virtual char *print( char *, int );

    /*
     * channel back-enable 
     */
    virtual void backenable(int iCtx, int iChan);

    /* interupt handler */
    virtual int interrupt(int iCtx)=0;

    /* timer handler */
    virtual int timer(int iCtx, const Time &)=0;


    void init(UmiChannel *pcn) { _cnuc.push(pcn); }
    void fini(UmiChannel *pcn) { _cnuc.remove(pcn); }
    UmiDriver(const char *szName, UmiCard *pca);
    virtual ~UmiDriver();
  };

  /*
   * Card abstraction
   */
  class UmiCard {
    ChainRaw _cnud;		/* all drivers		*/
    UmiDriver *_pudCur;		/* "current" driver	*/
   protected:
   public:
    UmiDriver *pudCur() { return _pudCur; }
    int setDriver(UmiDriver *pud) { if (_pudCur) return -1; _pudCur=pud; return 0; }
    int clrDriver(UmiDriver *pud) { if (_pudCur != pud) return -1; _pudCur=0; return 0; }

    void startIterDrivers( ChainIterator *pit) { _cnud.strtIter(pit); }
    UmiDriver *iterDrivers( ChainIterator *pit) { return static_cast<UmiDriver*>(_cnud.iter(pit)); }



    /* per card int and timer */
    virtual int interrupt(int iCtx)=0;
    virtual int timer(int iCtx, const Time &)=0;

    void initDriver(UmiDriver *pud) { _cnud.push(pud); }
    UmiCard():_pudCur(0) {}
    virtual ~UmiCard() {}
  };
}  


#endif


/* /home/INdep/src/gpib/umi.112/umi_user.hxx */
#ifndef _UMI_UMI_USER_HXX
#define _UMI_UMI_USER_HXX

/*
 * Universal Message Interface - User C++ part
 */

#ifndef EOF 
  #define EOF (-1)
#endif


namespace INdepUmi100 {



  /*
   * Universal Message Interface
   *
   * 'Failure' in the context of a Umi always means 'transportation' 
   * never 'endpoint' failure.
   */
  class Umi:public Failable {
    const UmiProto *_puf;
   protected:
    //char _mz[80];


    //UmiControl *setFailed() { _eStat=-1; return 0; }
    //UmiControl *setFailed(int e) { _eStat=e; return 0; }
    //int eStat() const { return _eStat; }
    const UmiProto *puf() { return _puf; }


   public:
    /*
     * transport failures
     */
    enum {
     OK,
     E_NOBUF
    };

    /*
     * send message
     */
    virtual Failable& putMsg(UmiControl*puc, const UmiData*pud) = 0;

    /*
     * get message
     */
    virtual Failable& getMsg(UmiControlBuf *puc, UmiData *pud) = 0;
    virtual Failable& xchMsg(UmiControlBuf *puc, UmiData *pud);

    /* flow control prevent deadlock */
    virtual bool canPut() ;


    /* data max per message */
    size_t sDataMax() { return 64; }		

    Umi(const UmiProto *puf);
    virtual ~Umi();
  };



  class Factory {
   public:
    /*
     * create channel to driver specified by protocol and channel number
     * (thread-safe)
     */
    virtual Failable createUmi(Umi**ppum, const UmiProto *, int iChan) const =0;

    Factory() {} 
    virtual ~Factory() {}
  };

  /* type of factory maker functions */
  typedef Factory* (*PFF) (const char *szMode, const char *szArg);

  /*
   * implementation dependent list of Factories
   * (thread-safe)
   */
  class Factories {
   public:

    /*
     * creates a new factory bound to a driver 
     * with the following properties:
     *
     * szMode = access domain (l)ocal, (k)ernel, (n)et
     * szArg = location interpreted by domain
     *
     * The factory created is able to connect to all
     * channels of the driver.
     * 
     * The factory must be deleted by the caller.
     */
    virtual Factory* puf(const char *szMode, const char *szArg) const = 0;

    Factories() {}
    virtual ~Factories() {}
  };

}
#endif


/* /home/INdep/src/gpib/umi.112/umi_c.h */
#ifndef _UMI_C_H_
#define _UMI_C_H_



  struct UmiIter {
     char _mpch[8*8];
  };

  struct UmiMemInfo {
    size_t _s;
    void * _p;
  };

  struct UmiCard ;
  struct UmiDriver ;
  struct UmiChannel ;
  struct UmiMsgDownBuf ;
  struct UmiMsgUpBuf ;
  struct CardType ;

  struct UmiEnvC {
    void (*_pFtiNextTick)(struct timespec *);
    void (*_pFsigGet)(int, void*);
    void (*_pFsigPut)(int, void*);
  };


  /*
   * module specifics methods
   */
  struct UmiMod {
    int _cChan;			/* channel per card		*/
    int _tps;			/* timer ticks per second	*/
    void *(*_pFnew_UmiCard)(struct UmiMod*, const struct UmiEnvC*, struct UmiMemInfo*, int, const struct CardType *, long lAddr);
    int (*_pFgetMem_UmiCard)(struct UmiMod*, struct UmiMemInfo*, int);
  };

  extern int UmiCard_timer( struct UmiCard*, int, const struct timespec *);
  extern int UmiCard_interrupt( struct UmiCard*, int );

  /*
   * generic methods
   */
  extern void UmiCard_startIterDrivers( struct UmiCard*, struct UmiIter *);
  extern struct UmiDriver * UmiCard_iterateDrivers( struct UmiCard*, struct UmiIter *);
  extern const char * UmiDriver_szName( struct UmiDriver *);
  extern char * UmiDriver_print( struct UmiDriver *, char *, int );

  extern void UmiDriver_startIterChannels( struct UmiDriver*, struct UmiIter *);
  extern struct UmiChannel* UmiDriver_iterateChannels( struct UmiDriver*, struct UmiIter *);
  extern int UmiChannel_num( struct UmiChannel *);
  extern int UmiChannel_open( struct UmiChannel *);
  extern const char* UmiChannel_szProto( struct UmiChannel *);
  extern void UmiChannel_close( struct UmiChannel *);
  extern void UmiChannel_setHandle( struct UmiChannel *, void *p);

  /*
   * channel upstream 
   */
  extern int UmiChannel_canget( struct UmiChannel *);
  extern struct UmiMsgUpBuf *UmiChannel_getUpBuf( struct UmiChannel *);			/* next to get		*/
  extern void UmiChannel_releaseUpBuf( struct UmiChannel *, int iCtx, struct UmiMsgUpBuf *);	/* last got		*/

  /*
   * channel downstream 
   */
  extern int UmiChannel_canput( struct UmiChannel *);
  extern struct UmiMsgDownBuf *UmiChannel_allocDownBuf( struct UmiChannel *);	/* next to put		*/
  extern void UmiChannel_putDownBuf( struct UmiChannel *, int iCtx, struct UmiMsgDownBuf *); /* last got		*/

  extern size_t UmiMsgUpBuf_sizeControl(struct UmiMsgUpBuf *);
  extern struct UmiControlStruct * UmiMsgUpBuf_pControl(struct UmiMsgUpBuf *);
  extern size_t UmiMsgUpBuf_sizeData(struct UmiMsgUpBuf *);
  extern char * UmiMsgUpBuf_pData(struct UmiMsgUpBuf *);

  extern size_t UmiMsgDownBuf_sizeControl(struct UmiMsgDownBuf *);
  extern size_t UmiMsgDownBuf_sizeDataMax(struct UmiMsgDownBuf *);
  extern void UmiMsgDownBuf_setDataLim(struct UmiMsgDownBuf *, size_t);
  extern struct UmiControlStruct * UmiMsgDownBuf_pControl(struct UmiMsgDownBuf *);

  extern char * UmiMsgDownBuf_pData(struct UmiMsgDownBuf *);

  #define UmiMod_cChan(pum) (pum->_cChan)
  #define UmiCard_new(pum, pue, a, b, c, d) ((*pum->_pFnew_UmiCard)((pum), (pue), (a), (b), (c), (d)))
  #define UmiCard_getMem(pum, a, b) ((*pum->_pFgetMem_UmiCard)(pum, (a), (b)))

  #define MAX_UMI_MEM 32

#endif
/* /home/INdep/src/gpib/umi.112/umi_c.hxx */

#ifndef _UMI_C_HXX_
#define _UMI_C_HXX_

extern "C" {
}


namespace INdepUmi100 {

  class UmiEnvRed : public UmiEnv {
    const UmiEnvC *_pueC;

   public:
    Time tiNext( const Time &ti );		
    void sigGet( int, void * );		
    void sigPut( int, void * );		

    UmiEnvRed(const UmiEnvC *);
    ~UmiEnvRed();
  };

}



#endif
/* /home/INdep/src/gpib/gpib.112/regs.hxx */

#ifndef _REGS_HXX_
#define _REGS_HXX_ 1


#if (MMSYS_wdm)
extern "C" { 
  /*
   * WinNT Kernel specifics
   */
} 
#endif
#if (MMSYS_lkern)
  /*
   * Linux Kernel specifics
   */
  static __inline unsigned char inb (unsigned short int port)
  {
    unsigned char _v;

    __asm__ __volatile__ ("inb %w1,%0":"=a" (_v):"Nd" (port));
    return _v;
  }

  static __inline void outb (unsigned char value, unsigned short int port)
  {
    __asm__ __volatile__ ("outb %b0,%w1": :"a" (value), "Nd" (port));
  }

  extern int printk( const char *fmt, ...);
  #define PREFIX "<0>core488: "
#endif

  /*
   * Registers
   */

namespace INdepGpib100 {
  class Regs {
    int _ioBase;

  #if (MMSYS_wdm)	/* WinNT kernel */
   public:
    /* for WDM driver */
    int read(int iReg) { return READ_PORT_UCHAR((PUCHAR)_ioBase+iReg); }
    int readByte(int iReg) { return READ_PORT_UCHAR((PUCHAR)_ioBase+iReg); }
    void write(int iReg, int i) { WRITE_PORT_UCHAR((PUCHAR)_ioBase+iReg, (UCHAR)i); }
    bool isGood() const { return true; }

    Regs(int ioBase):_ioBase(ioBase) {}
    ~Regs() {}

  #endif 

  #if (defined(MMSYS_linux) || defined(MMSYS_mingw) || MMSYS_win)
    
    //int _fdMem;
    void *_pMemIo;
    volatile void *pMemIo() { return static_cast<volatile void *>(_pMemIo); }


   public:
    //bool isGood() const ;
    bool isGood() const { return true; }
    int read(int iReg) { return static_cast<volatile int *>(pMemIo())[iReg] & 0xff; }
    void write(int iReg, int i) { static_cast<volatile int *>(pMemIo())[iReg] = i; }
    int readByte(int iReg) { return static_cast<volatile unsigned char *>(pMemIo())[iReg] & 0xff; }

    //Regs(int iAddrMem);
    //~Regs();

    Regs(int ioBase):_pMemIo((void*) ioBase) {}
    ~Regs() {}
  #endif 
    

  #ifdef MMSYS_lkern	/* linux kernel */
   public:
    bool isGood() const { return true; }
    int read(int iReg) { return inb(_ioBase+iReg); }
    int readByte(int iReg) { return inb(_ioBase+iReg); }
    void write(int iReg, int i) { outb(i, _ioBase+iReg); }

    Regs(int ioBase):_ioBase(ioBase) {}
    ~Regs() {}

  #endif 

  };
} 


#endif
/* /home/INdep/src/gpib/gpib.112/gpib.h */
/************************************************************

  Declarations from the IEEE488.2 standard

************************************************************/

#ifndef _BASE_GPIB_H
#define _BASE_GPIB_H

/* GPIB interface commands */
#define	GPIB_GTL	   0x01	/* goto local */
#define GPIB_SDC	   0x04	/* selective device clear */
#define	GPIB_PPC	   0x05	/* parallel poll configure */
#define GPIB_GET	   0x08	/* group execute trigger */
#define GPIB_TCT	   0x09	/* take control */
#define GPIB_LLO	   0x11	/* local lockout */
#define GPIB_DCL	   0x14	/* generic device clear */
#define GPIB_PPU	   0x15	/* parallel poll unconfigure */
#define GPIB_SPE	   0x18	/* serial poll enable */
#define GPIB_SPD	   0x19	/* serial poll disable */
#define GPIB_LAG	   0x20	/* listener address group */
#define GPIB_UNL	   0x3f	/* unlisten */
#define GPIB_TAG	   0x40	/* talker address group */
#define GPIB_UNT	   0x5f	/* untalk */
#define GPIB_SCA	   0x60	/* secondary address group */
#define GPIB_PPE	   0x60	/* parallel poll enable */
#define GPIB_PPD	   0x70	/* parallel poll disable */

/* IEEE-488.2 Status Byte register bits */
#define GPIB_4882_STB_MSS  0x40
#define GPIB_4882_STB_RQS  0x40
#define GPIB_4882_STB_ESB  0x20
#define GPIB_4882_STB_MAV  0x10

/* IEEE-488.2 Standard Event Status Register bits */

#define GPIB_4882_ESR_PON  0x80
#define GPIB_4882_ESR_URQ  0x40
#define GPIB_4882_ESR_CME  0x20
#define GPIB_4882_ESR_EXE  0x10
#define GPIB_4882_ESR_DDE  0x08
#define GPIB_4882_ESR_QYE  0x04
#define GPIB_4882_ESR_RQC  0x02
#define GPIB_4882_ESR_OPC  0x01

#endif 
/* /home/INdep/src/gpib/gpib.112/arch.h */
#ifndef _GPIB_GPIB_ARCH_H_
#define _GPIB_GPIB_ARCH_H_ 1


  /* GPIB controller architectures */

  enum GcArchE {			/* controller chip register architectures */
	  GCA_7210,
	  GCA_72010,
	  GCA_72130,
	  GCA_9914,
	  CNT_GCA
  };
  extern const char *mpszGcArch[CNT_GCA];
  #define szGcArch(i) mpszGcArch[(i)]
  #define GCA_GP5130 GCA_72130



#endif 

/* /home/INdep/src/gpib/gpib.112/cards.h */
#ifndef _GPIB_GPIB_CARDS_H_
#define _GPIB_GPIB_CARDS_H_ 1



#define SZ_CARD_ID 32
#define MAX_PCI_REGION 2
#define MAX_REGION MAX_PCI_REGION 

#ifdef __cplusplus 
extern "C" {
#endif

  enum GcType {
    CT_GPIB_PCI_XL,
    CT_GPIB_PCI,
    CT_GPIB_PC104,
    CT_GPIB_PCMCIA,
    CT_GPIB_PC,
    CNT_CARDTYPES
  };

  /*
   * Card types
   */
  struct RegionDesc {
    int _iBar;			/* PCI bar offset  		*/
    int _f;
    int _l;			/* region length in bytes	*/
  };

  #define RG_AMCC5920_CONF 001	/* AMCC local config region	*/
  #define RG_IOBASE 002		/* GPIB register region		*/
  #define RG_IOMEM 004		/* GPIB memory region		*/
  #define RG_ISMEM(x) (((x) & RG_IOMEM)!=0)

  struct CardType {
    int _eCard;			/* enumerates all known cards	*/
    const char *_szName;
    const char *_szArch; 	/* controller chip architecture */
    enum GcArchE _eArch;	/* architecture enumerated	*/
    
    int _eBus;			/* bus type	*/
    int _isaLen;		/* ISA only: size of IO region */

    /* pci regions */
    struct RegionDesc _mprd[MAX_PCI_REGION];
  };
  extern const struct CardType mpct[CNT_CARDTYPES+1];
  extern const struct RegionDesc *getRegionBySize(const struct CardType *, int len);

#ifdef __cplusplus 
}
#endif

#endif 

/* /home/INdep/src/gpib/gpib.112/igcregs.h */
/************************************************************

  Declarations for the various GPIB controllers

************************************************************/

#ifndef _IGPIBREG_H
#define _IGPIBREG_H


/* register bank 0; NEC compatible registers */
#define IGC_BANK0             0	/* start of register bank 0 */
#define IGC_DIO               0	/* (r/w) data in/ byte out */
#define IGC_I1	              1	/* (r/w) interrupt status/mask 1 */
#define IGC_I2	              2	/* (r/w) interrupt status/mask 2 */
#define IGC_SP	              3	/* (r/w) serial poll status/mode */
#define IGC_ADSTS             4	/* (r) address status */
#define IGC_ADMOD             4	/* (w) address mode */
#define IGC_CPT	              5	/* (r) Command pass through */
#define IGC_AUXMOD            5	/* (w) auxiliary mode */
#define IGC_AD0	              6	/* (r) address 0 */
#define IGC_AD01              6	/* (w) address 0/1 */
#define IGC_AD1	              7	/* (r) address 1 */
#define IGC_EOS1              7	/* (w) EOS1 byte */

/* register bank 1 */
#define IGC_BANK1             8	/* start of register bank 1 */
#define IGC_FFSTS             8	/* (r) FIFO status */
#define IGC_XDMA              8	/* (w) extended DMA control */
#define IGC_I3	              9	/* (r/w) interrupt status/mask 3 */
#define IGC_I4	             10	/* (r/w) interrupt status/mask 4 */
#define IGC_TCNTLO           11	/* (r/w) terminal count low byte */
#define IGC_TCNTHI           12	/* (r/w) terminal count high byte */
#define IGC_REVC             13	/* (r) revision code */
#define IGC_TMOLO            13	/* (w) timeout low byte */
#define IGC_OEMC             14	/* (r) oem code */
#define IGC_TMOHI            14	/* (w) timeout high byte */
#define IGC_XSTS             15	/* (r) extended status */
#define IGC_XMOD             15	/* (w) extended mode */

/* register bank 2 */
#define IGC_BANK2            16	/* start of register bank 2 */
#define IGC_IFFCNT           16	/* (r) input FIFO count */
#define IGC_IFFWM            16	/* (w) input FIFO watermark */
#define IGC_OFFCNT           17	/* (r) output FIFO count */
#define IGC_OFFWM            17	/* (w) output FIFO watermark */
#define IGC_BMDTA            18	/* (r/w) bus monitor data */
#define IGC_BMCTL            19	/* (r/w) bus monitor ctrl */
#define IGC_BMDIR            20	/* (w) bus monitor direction */
#define IGC_SRQEN            21	/* (w) srq enable */
#define IGC_TSTRG            22	/* (r) test register */
#define IGC_EOS2             22	/* (w) EOS 2 byte */
#define IGC_EOS3             23	/* (w) EOS 3 byte */

/* interrupt register 1 */
#define IGC_I1_DI	   	   0x01	/* data in */
#define IGC_I1_DO		   0x02	/* data out */
#define IGC_I1_ERR	   0x04	/* error: byte out cont. lost */
#define IGC_I1_DEC	   0x08	/* device clear */
#define IGC_I1_END	   0x10	/* end message */
#define IGC_I1_DET	   0x20	/* device trigger */
#define IGC_I1_APT	   0x40	/* address pass through */
#define IGC_I1_CPT	   0x80	/* command pass through */

/* interrupt register 2 */
#define IGC_I2_ADSC	   0x01	/* address changed */
#define IGC_I2_REMC	   0x02	/* remote changed */
#define IGC_I2_LOKC	   0x04	/* lok changed */
#define IGC_I2_CO   	   0x08	/* command out */
#define IGC_I2_REM 	   0x10	/* remote (r) */
#define IGC_I2_DMAI	   0x10	/* DMA on input (w) */
#define IGC_I2_LOK 	   0x20	/* lokal (r) */
#define IGC_I2_DMAO	   0x20	/* DMA on output (w) */
#define IGC_I2_SRQI	   0x40	/* service request */
#define IGC_I2_INT	   0x80	/* any of these interrupts */

/* extended interrupt register 3 */
#define IGC_I3_TMO	   0x01	/* timer expiration */
#define IGC_I3_TC	     	   0x02	/* terminal count */
#define IGC_I3_CMD	   0x04	/* command received */
#define IGC_I3_TCT	   0x08	/* take control received */
#define IGC_I3_IFC	   0x10	/* IFC has been set */
#define IGC_I3_ATN	   0x20	/* atn has been set */
#define IGC_I3_FFERR	   0x40	/* error on FIFO */

/* extended interrupt register 4 */
#define IGC_I4_IFWM	   0x01	/* input FIFO reached watermark */
#define IGC_I4_OFWM	   0x02	/* output FIFO reached watermark */
#define IGC_I4_IFFL	   0x04	/* input FIFO full */
#define IGC_I4_OFEM	   0x08	/* output FIFO empty */
#define IGC_I4_IFRDY	   0x10	/* input FIFO ready */
#define IGC_I4_OFRDY	   0x20	/* output FIFO ready */
#define IGC_I4_NIFWM	   0x40	/* input FIFO under watermark */
#define IGC_I4_NOFWM	   0x80	/* output FIFO under watermark */

/* igpib auxiliary commands (IGC_AUX) */
#define IGC_AUXCMD_IPO	   0x00 /* immideate exectute pon */
#define IGC_AUXCMD_RES	   0x02	/* chip reset */
#define IGC_AUXCMD_FH	   0x03	/* finish handshake */
#define IGC_AUXCMD_TRG	   0x04	/* trigger (pulse trig pin) */
#define IGC_AUXCMD_RTL	   0x05	/* return to local (pulse mode ) */
#define IGC_AUXCMD_SRTL	   0x0d	/* return to local (set 'rtl') */
#define IGC_AUXCMD_FEOI	   0x06	/* send EOI */
#define IGC_AUXCMD_ANV	   0x07	/* address not valid */
#define IGC_AUXCMD_AV	   0x0f	/* address valid */
#define IGC_AUXCMD_PPFC	   0x01	/* clear ppoll flag: use srq */
#define IGC_AUXCMD_PPFS	   0x09	/* set ppoll flag: use ppoll */
#define IGC_AUXCMD_GTS	   0x10	/* go to standby */
#define IGC_AUXCMD_TCA	   0x11	/* take control async */
#define IGC_AUXCMD_TCS	   0x12	/* take control sync */
#define IGC_AUXCMD_TCE	   0x1a	/* take control sync on end */
#define IGC_AUXCMD_LTN	   0x13	/* listen, gen 'ltn' pulse */
#define IGC_AUXCMD_LTC	   0x1b	/* listen in continous mode */
#define IGC_AUXCMD_LUN	   0x1c	/* local unlisten */
#define IGC_AUXCMD_EPP	   0x1d	/* execute parallel poll */
#define IGC_AUXCMD_CIFC	   0x16	/* clear ifc */
#define IGC_AUXCMD_CREN	   0x17	/* clear remote enable */
#define IGC_AUXCMD_GOTMS   0x18 /* goto TMS mode */
#define IGC_AUXCMD_SIFC    0x1e /* set ifc */
#define IGC_AUXCMD_SREN	   0x1f	/* set remote enable */
#define IGC_AUXCMD_DSC	   0x14	/* disable system control */
#define IGC_AUXCMD_XMOD	   0x0a	/* switch to extended mode */

/* auxuliary register offsets */
#define IGC_AUXOFF_CLK	   0x20	/*  refclock */
#define IGC_AUXOFF_AUXD	   0x40	/*  write to aux D */
#define IGC_AUXOFF_WPP	   0x60	/*  write ppol reg */
#define IGC_AUXOFF_AUXA	   0x80	/*  write aux A */
#define IGC_AUXOFF_AUXB	   0xa0	/*  write aux B */
#define IGC_AUXOFF_AUXE	   0xc0	/*  write aux E */
#define IGC_AUXOFF_AUXF	   0xe0	/*  write ext. aux reg */

/* auxiliary register A */
#define IGC_AUXA_NHS	   0x00	/* normal handshake mode */
#define IGC_AUXA_HAM	   0x01	/* rfd holdoff on all data */
#define IGC_AUXA_HEM	   0x02	/* rfd holdoff on end mode */
#define IGC_AUXA_CM	   0x03	/* continous mode */
#define IGC_AUXA_LEOS	   0x04	/* enable end at eos (listen) */
#define IGC_AUXA_TEOS	   0x08	/* enable end at eos (talk) */
#define IGC_AUXA_EOSB	   0x10	/* enable 8-Bit eos detection */

/* auxiliary register B */
#define IGC_AUXB_UDCPT	   0x01	/* set CPT during detection of undef cmds */
#define IGC_AUXB_SPEND	   0x02	/* permit the transm. of end during spoll */
#define IGC_AUXB_HISP	   0x04	/* highspeed on write data */
#define IGC_AUXB_INTLVL	   0x08	/* set active level of int pin */
#define IGC_AUXB_ISTSRQ	   0x10	/* set ist to SRQ ppoll otherwise */

/* aux register E */
#define IGC_AUXE_CLR	   0x01	/* holdoff on clear */
#define IGC_AUXE_TRG	   0x02	/* holdoff on trigger */
#define IGC_AUXE_ALL	   0x04	/* holdoff on all cmds */
#define IGC_AUXE_TCT	   0x08	/* holdoff on take control */

/* aux register F */
#define IGC_AUXF_DISL      0x00 /* disable listener function */
#define IGC_AUXF_ENAL	   0x01	/* enable listener function */
#define IGC_AUXF_MAP1	   0x02	/* map registerset 1 on 0 */
#define IGC_AUXF_MAP2	   0x03	/* map registerset 2 on 0 */
#define IGC_AUXF_RFDH	   0x04	/* set RFD Holdoff */
#define IGC_AUXF_OFFCLR	   0x05	/* output FIFO counter clear */
#define IGC_AUXF_IFFCLR	   0x06	/* input FIFO counter clear */

/* address status register (IGC_ADSTS) */
#define IGC_ADSTS_MJMN	   0x01	/* major/minor address */
#define IGC_ADSTS_TA	   0x02	/* talker activer state */
#define IGC_ADSTS_LA	   0x04	/* listener active */
#define IGC_ADSTS_TPAS	   0x08	/* talker primary addressed state */
#define IGC_ADSTS_LPAS	   0x10	/* listener primary addressed state */
#define IGC_ADSTS_SPMS	   0x20	/* serial poll mode state */
#define IGC_ADSTS_NATN	   0x40	/* attention */
#define IGC_ADSTS_CIC	   0x80	/* controller in charge */

/* address mode register (IGC_ADMOD) */
#define IGC_ADMOD_ADM0	   0x01	/* addr mode minor bit */
#define IGC_ADMOD_ADM1	   0x02	/* addr mode major bit */
#define IGC_ADMOD_OFENA	   0x04	/* output FIFO enable (256 byte) */
#define IGC_ADMOD_IFENA	   0x08	/* input FIFO enable */
#define IGC_ADMOD_TRM0	   0x10	/* transmit/receive mode 0 */
#define IGC_ADMOD_TRM1	   0x20	/* transmit/receive mode 1 */
#define IGC_ADMOD_LON	   0x40	/* listen only */
#define IGC_ADMOD_TON	   0x80	/* talk only */

/* address register 0 (IGC_AD0) */
#define IGC_AD0_DLAD	   0x20	/* (r/w) disable listener address */
#define IGC_AD0_DTAD	   0x40	/* (r/w) disable talker address */
#define IGC_AD0_ADSEL	   0x80	/* (w) address register select */
#define IGC_AD0_T8RUN	   0x80	/* (r) set by ifc/ren and cleared by timer */

/* address register 1 (IGC_AD1) */
#define IGC_AD1_DLAD	   0x20	/* (r) disable listener address */
#define IGC_AD1_DTAD	   0x40	/* (r) disable talker address */
#define IGC_AD1_EOI	   0x80	/* (r) latched eoi of current byte in FIFO */

/* extended DMA control (IGC_XDMA) */
#define IGC_XDMA_OUT       0x01	/* extended DMA output (GPIB talk) */
#define IGC_XDMA_ENA	   0x02	/* extended DMA enable */
#define IGC_XDMA_SPD	   0x04	/* DMA speed set-> IO on every clock tick
				   else on every 2. clock tick */
#define IGC_XDMA_AUXO	   0x08	/* DMA width; set 16-Bit */

/* extended status register (IGC_XSTS) */
#define IGC_XSTS_OQMU	   0x01	/* output queue message unit in prog */
#define IGC_XSTS_SCSEL	   0x02	/* SCSEL pin on 68 pin chip */
#define IGC_XSTS_LIDIS	   0x04	/* listener mode disabled */
#define IGC_XSTS_IFEM	   0x08	/* input FIFO is empty */
#define IGC_XSTS_OFFL	   0x10	/* output FIFO is full */

/* extended mode register (IGC_XMOD) */
#define IGC_XMOD_TRGEN	   0x01	/* enable trigger on T/R3 Pin */
#define IGC_XMOD_MAVEN	   0x02	/* reset MAV on end */
#define IGC_XMOD_E1ENA	   0x04	/* enable extended EOS1 */
#define IGC_XMOD_E2ENA	   0x08	/* enable extended EOS2 */
#define IGC_XMOD_EOIDIS	   0x10	/* ignore eoi */
#define IGC_XMOD_TCENA	   0x20	/* enable terminal count */
#define IGC_XMOD_TCTLK	   0x40	/* use terminal count on talk */
#define IGC_XMOD_TCLBH	   0x80	/* enable last byte handling */

/* bus monitor control lines */
#define IGC_BMCTL_DAV	   0x01	/* data valid */
#define IGC_BMCTL_NRFD	   0x02	/* ready for data */
#define IGC_BMCTL_NDAC	   0x04	/* data accepted */
#define IGC_BMCTL_IFC	   0x08	/* ifc line */
#define IGC_BMCTL_ATN	   0x10	/* atn line */
#define IGC_BMCTL_SRQ	   0x20	/* srq line */
#define IGC_BMCTL_REN	   0x40	/* remote line */
#define IGC_BMCTL_EOI	   0x80	/* eoi line */

/* test register */
#define IGC_TSTRG_SRQS	   0x01	/* service request state */

/* bus monitor write control */
#define IGC_BMDIR_DTA	   0x01	/* set to data as output */
#define IGC_BMDIR_SC	   0x02	/* system controller */
#define IGC_BMDIR_EOIOE	   0x04	/* set EOI as output */
#define IGC_BMDIR_CIC	   0x08
#define IGC_BMDIR_PE	   0x10
#define IGC_BMDIR_ENA	   0x80	/* enable for output */


/* TMS kompatible registers, bits and commands */

#define TMS_I0                0	/* (r/w) interrupt status/mask 1*/
#define TMS_I1	              1	/* (r/w) interrupt status/mask 2 */
#define TMS_ADSTS	      2	/* (r) address status */
#define TMS_BUS	              3	/* (r) bus status */
#define TMS_AUXMOD            3	/* (w) auxiliary mode */
#define TMS_AD	              4	/* (w) address mode */
#define TMS_SP                5	/* (w) seriall poll */
#define TMS_CPT	              6	/* (r) Command pass through */
#define TMS_PP                6	/* (w) parallel poll */
#define TMS_DIO	              7	/* (r/w) data in/out */

/* interrupt register 1 */
#define TMS_I0_MAC	   0x01	/* my address change */
#define TMS_I0_RLC	   0x02	/* remote local change */
#define TMS_I0_SPAS	   0x04	/* serial poll sent */
#define TMS_I0_END	   0x08	/* end message */
#define TMS_I0_BO	   0x10 /* data out */
#define TMS_I0_BI	   0x20	/* data in */
#define TMS_I0_INT1  	   0x40	/* interrupt in INT1 */
#define TMS_I0_INT0	   0x80	/* interrupt in INT0 */

/* interrupt register 2 */
#define TMS_I1_IFC	   0x01	/* interface clear received */
#define TMS_I1_SRQ	   0x02	/* service request received */
#define TMS_I1_MA	   0x04	/* My Address rec.*/
#define TMS_I1_DCAS	   0x08	/* device clear active state */
#define TMS_I1_APT 	   0x10	/* Address pass through */
#define TMS_I1_UNC 	   0x20	/* unrecognized command */
#define TMS_I1_ERR	   0x40	/* No listener found */
#define TMS_I1_GET	   0x80	/* Trigger received */

/* address status register (TMS_ADSTS) */
#define TMS_ADSTS_ULPA	   0x01	/* LSB of last recog. address */
#define TMS_ADSTS_TADS	   0x02	/* talker addressed/active state */
#define TMS_ADSTS_LADS	   0x04	/* listener addr./active state */
#define TMS_ADSTS_TPAS	   0x08	/* talker primary addressed state */
#define TMS_ADSTS_LPAS	   0x10	/* listener primary addressed state */
#define TMS_ADSTS_ATN	   0x20	/* attention is true */
#define TMS_ADSTS_LLO	   0x40	/* lockout in operation */
#define TMS_ADSTS_REM	   0x80	/* device is in remote state */

/* address mode register */
#define TMS_AD_DTAD	   0x20	/* (r/w) disable talker address */
#define TMS_AD_DLAD	   0x40	/* (r/w) disable listener address */
#define TMS_AD_EDPA	   0x80	/* enable dual primary addr mode */

/* auxiliary command register */
#define TMS_AUXCMD_TRUE	   0x80	/* set option to true */

#define TMS_AUXCMD_SWRES   0x00 /* Software reset */
#define TMS_AUXCMD_DACR	   0x01	/* release DAC holdoff */
#define TMS_AUXCMD_RHDF	   0x02	/* release RFD holdoff */
#define TMS_AUXCMD_HDFA	   0x03	/* holdoff on all data */
#define TMS_AUXCMD_HDFE	   0x04	/* holdoff on EOI only */
#define TMS_AUXCMD_NBAF	   0x05	/* new byte avaliable false */
#define TMS_AUXCMD_FGET	   0x06	/* force group execute trigger */
#define TMS_AUXCMD_RTL	   0x07	/* return to local */
#define TMS_AUXCMD_FEOI	   0x08	/* send EOI with next byte */
#define TMS_AUXCMD_LON	   0x09	/* listen only */
#define TMS_AUXCMD_TON	   0x0a	/* talk only */
#define TMS_AUXCMD_GTS	   0x0b	/* go to standby */
#define TMS_AUXCMD_TCA	   0x0c	/* Take control asynchronously */
#define TMS_AUXCMD_TCS	   0x0d	/* take control synchronously */
#define TMS_AUXCMD_RPP	   0x0e	/* request parallel poll */
#define TMS_AUXCMD_SIC	   0x0f	/* send interface clear */
#define TMS_AUXCMD_SRE	   0x10	/* send remote enable */
#define TMS_AUXCMD_RQC	   0x11	/* request control */
#define TMS_AUXCMD_RLC	   0x12	/* release control */
#define TMS_AUXCMD_DAI	   0x13	/* disable all interrupts */
#define TMS_AUXCMD_PTS	   0x14	/* pass through next secondary */
#define TMS_AUXCMD_STDL	   0x15	/* short t1 settling time */
#define TMS_AUXCMD_SHDW	   0x16	/* shadow handshake */
#define TMS_AUXCMD_VSTDL   0x17	/* very short T1 delay */
#define TMS_AUXCMD_RSV2	   0x18	/* request service bit 2 */

#define TMS_BUS_REN	   0x01	/* remote line */
#define TMS_BUS_IFC	   0x02	/* ifc line */
#define TMS_BUS_SRQ	   0x04	/* srq line */
#define TMS_BUS_EOI	   0x08	/* eoi line */
#define TMS_BUS_NRFD	   0x10	/* ready for data */
#define TMS_BUS_NDAC	   0x20	/* data accepted */
#define TMS_BUS_DAV	   0x40	/* data valid */
#define TMS_BUS_ATN	   0x80	/* atn line */

/* GP5130 >= 1.5 */

/*
 * 5130 register set
 */
#define GP5130_I1      	   IGC_I1
#define GP5130_I2      	   IGC_I2
#define GP5130_AD1     	   IGC_AD1
#define GP5130_DIO     	   IGC_DIO
#define GP5130_AUXMOD      IGC_AUXMOD
#define GP5130_RFFLO       0x08  // (r) read fifo count lo byte
#define GP5130_RTCLO       0x08  // (w) read transfer count lo byte
#define GP5130_RXSTS       0x09  // (r) extended read status
#define GP5130_RXMOD       0x09  // (w) extended read mode
#define GP5130_WFFLO       0x0a  // (r) write fifo count lo byte
#define GP5130_WTCLO       0x0a  // (w) write transfer count lo byte
#define GP5130_WXMOD       0x0b  // (w) extended write mode
#define GP5130_WXSTS       0x0b  // (r) extended write status
#define GP5130_GPCIN       0x0c  // (r) gpib control lines (inputs)
#define GP5130_GPCOUT	   0x0d  // (r) gpib control lines (outputs)
#define GP5130_VERS	   0x0e  // (r) version register
#define GP5130_XSTS	   0x0f  // (r) extended status

/* 
 * 5130 auxiliary commands 
 */
#define GP5130_AUXCMD_FH   IGC_AUXCMD_FH	 
#define GP5130_AUXCMD_XMOD IGC_AUXCMD_XMOD


// auxiliary register offsets
//---------------------------
#define GP5130_AUXOFF_CLK	 0x20	//  refclock
#define GP5130_AUXOFF_AUXD 0x40	//  write to aux D
#define GP5130_AUXOFF_WPP	 0x60	//  write ppol reg
#define GP5130_AUXOFF_AUXA 0x80	//  write aux A
#define GP5130_AUXOFF_AUXB 0xa0	//  write aux B
#define GP5130_AUXOFF_AUXE 0xc0	//  write aux E
#define GP5130_AUXOFF_AUXF 0xe0	//  write ext. aux reg

// auxiliary register A
//---------------------
#define GP5130_AUXA_NHS	   0x00	// normal handshake mode
#define GP5130_AUXA_HAM	   0x01	// rfd holdoff on all data
#define GP5130_AUXA_HEM	   0x02	// rfd holdoff on end mode
#define GP5130_AUXA_CM	   0x03	// continous mode
#define GP5130_AUXA_LEOS	 0x04	// enable end at eos (listen)
#define GP5130_AUXA_TEOS	 0x08	// enable end at eos (talk)
#define GP5130_AUXA_EOSB	 0x10	// enable 8-Bit eos detection

// auxiliary register B
//---------------------
#define GP5130_AUXB_UDCPT	 0x01	// set CPT during detection of undef cmds
#define GP5130_AUXB_SPEND	 0x02	// permit the transm. of end during spoll
#define GP5130_AUXB_HISP	 0x04	// highspeed on write data
#define GP5130_AUXB_INTLVL 0x08	// set active level of int pin
#define	GP5130_AUXB_ISTSRQ 0x10	// set ist to SRQ ppoll otherwise

#if 0
// aux register E
#define GP5130_AUXE_CLR	   0x01	// holdoff on clear
#define GP5130_AUXE_TRG	   0x02	// holdoff on trigger
#define GP5130_AUXE_ALL	   0x04	// holdoff on all cmds
#define GP5130_AUXE_TCT	   0x08	// holdoff on take control

// aux register F
#define GP5130_AUXF_DISL	 0x00	// disable listener function
#define GP5130_AUXF_ENAL	 0x01	// enable listener function
#define GP5130_AUXF_MAP1	 0x02	// map registerset 1 on 0
#define GP5130_AUXF_MAP2	 0x03	// map registerset 2 on 0
#define GP5130_AUXF_RFDH	 0x04	// set RFD Holdoff
#define GP5130_AUXF_OFFCLR 0x05	// output FIFO counter clear
#define GP5130_AUXF_IFFCLR 0x06	// input FIFO counter clear
#endif //0


// address status register
//------------------------
#define GP5130_ADSTS_MJMN	 0x01	// major/minor address
#define GP5130_ADSTS_TA	   0x02	// talker activer state
#define GP5130_ADSTS_LA	   0x04	// listener active
#define GP5130_ADSTS_TPAS	 0x08	// talker primary addressed state
#define GP5130_ADSTS_LPAS	 0x10	// listener primary addressed state
#define GP5130_ADSTS_SPMS	 0x20	// serial poll mode state
#define GP5130_ADSTS_NATN	 0x40	// attention
#define GP5130_ADSTS_CIC	 0x80	// controller in charge

// address mode register
//----------------------
#define GP5130_ADMOD_ADM0	 0x01	// addr mode minor bit
#define GP5130_ADMOD_ADM1	 0x02	// addr mode major bit
//#define	GP5130_ADMOD_OFENA0x04	// output FIFO enable (256 byte)
//#define GP5130_ADMOD_IFENA0x08	// input FIFO enable
#define GP5130_ADMOD_TRM0	 0x10	// transmit/receive mode 0
#define GP5130_ADMOD_TRM1	 0x20	// transmit/receive mode 1
#define GP5130_ADMOD_LON	 0x40	// listen only
#define GP5130_ADMOD_TON	 0x80	// talk only

// address register 0
//-------------------
#define GP5130_AD0_DLAD	   0x20	// (r/w) disable listener address
#define GP5130_AD0_DTAD	   0x40	// (r/w) disable talker address
#define GP5130_AD0_ADSEL	 0x80	// (w) address register select
//#define GP5130_AD0_T8RUN 0x80	// (r) set by ifc/ren and cleared by timer

// address register 1
//-------------------
#define GP5130_AD1_DLAD	   0x20	// (r) disable listener address
#define GP5130_AD1_DTAD	   0x40	// (r) disable talker address
#define GP5130_AD1_EOI	   0x80	// (r) latched eoi of current byte in FIFO


// gpib control lines (GPCIN/GPCOUT)
//----------------------------------
#define GP5130_GPC_NRFD    0x01
#define GP5130_GPC_NDAC    0x02
#define GP5130_GPC_DAV     0x04
#define GP5130_GPC_EOI     0x08
#define GP5130_GPC_SRQ     0x10
#define GP5130_GPC_ATN     0x20
#define GP5130_GPC_REN     0x40
#define GP5130_GPC_IFC     0x80


// extended read/write mode/status
//--------------------------------
#define GP5130_XRW_ENTC    0x10 // (w)   enable transfer count
#define GP5130_XRW_FFCH    0x01 // (r)   fifo count bit 9
#define GP5130_XINT_TC     0x20 // (r/w) transfer count reached int bit/mask
#define GP5130_XINT_EMPT   0x40 // (r/w) fifo empty int bit/mask
#define GP5130_XINT_FULL   0x80 // (r/w) fifo full int bit/mask


// extended mode/status
//---------------------
#define GP5130_XMOD_HDF    0x01 // (w)   set RFD holdoff
#define GP5130_XSTS_RDY    0x01 // (r)   internal 'ready' msg
#define GP5130_XSTS_RTC    0x02 // (r)   read terminal count reached
#define GP5130_XSTS_REMPT  0x04 // (r)   read fifo empty
#define GP5130_XSTS_RFULL  0x08 // (r)   read fifo full
#define GP5130_XSTS_NBA    0x10 // (r)   internal 'new byte available' msg
#define GP5130_XSTS_WTC    0x20 // (r)   write terminal count reached
#define GP5130_XSTS_WEMPT  0x40 // (r)   write fifo empty
#define GP5130_XSTS_WFULL  0x80 // (r)   write fifo full


/* NI IBLINES */
#define  IBLINE_EOI   0x0080
#define  IBLINE_ATN   0x0040
#define  IBLINE_SRQ   0x0020
#define  IBLINE_REN   0x0010
#define  IBLINE_IFC   0x0008
#define  IBLINE_NRFD  0x0004
#define  IBLINE_NDAC  0x0002
#define  IBLINE_DAV   0x0001

#endif /* _IGCREGS_H */
/* /home/INdep/src/gpib/gpib.112/gumi.hxx */

#ifndef _GPIB_GUMI_HXX
#define _GPIB_GUMI_HXX


namespace INdepGpib100 {
  /* controller interface */
  enum ErrE {
    E_OK = 0,
    E_HWFAIL,		/* unexpected state at hardware layer 	*/
    E_ADR,		/* addressing scheme wrong		*/
    E_SAC,		/* system control required		*/
    E_NCIC,		/* interface not controller in charge	*/
    E_CIC,		/* device function called while CIC	*/
    E_TMO,		/* timeout				*/
    E_STATE,		/* wrong state				*/
    E_MEM,		/* allocation failure			*/
    E_TRANSFAIL,	/* prev. msg. of transaction failed 	*/
    E_INUSE,		/* interface already in use	 	*/
    E_MSG,		/* incompatible message format          */
    E_NOL,		/* no listeners detected		*/
    E_DCL,		/* DCAS during data transfer, clr E_DCAS */

    CNT_E
  };

  /* status enum */
  enum StatE {
    /* direct bits */
    F_LA =  0x01,	/* LAxS					*/
    F_TA =  0x02,	/* TAxS					*/
    F_CIC = 0x04,	/* CACS|CSBS				*/
    F_ATN = 0x08,	/* qualifies the flags above		*/
    F_SC =  0x10,	/* SACS					*/
    F_REM = 0x20,	/* REMS|RWLS				*/
    F_LOK = 0x40,	/* LWLS|RWLS				*/

    /* event bits */
    E_SRQI = 0x100,	/* SRQ seen				*/
    E_DTAS = 0x200,	/* DTAS seen				*/
    E_DCAS = 0x400	/* DCAS seen				*/
  };



  /* lines enum */
  enum LineE {
    L_NRFD    = 0x01,
    L_NDAC    = 0x02,
    L_DAV     = 0x04,
    L_EOI     = 0x08,
    L_SRQ     = 0x10,
    L_ATN     = 0x20,
    L_REN     = 0x40,
    L_IFC     = 0x80
  };
}


#endif 

/* /home/INdep/src/gpib/gpib.112/gumi_user.hxx */

#ifndef _GPIB_GUMI_USER_HXX
#define _GPIB_GUMI_USER_HXX


namespace INdepGpib100 {
  namespace Umi=INdepUmi100;
}


#endif 

/* /home/INdep/src/gpib/gpib.112/gumi_msgs.hxx */
/*
 * This file is automatically created
 * DO NOT EDIT
 */

class xistream;
class xostream;

#if (defined(MMCCS_msc) || defined(MMSYS_mingw))
  typedef unsigned u_int;		/* REMIND */
  typedef unsigned size_t;		/* REMIND */
#endif
namespace Umi=INdepUmi100;

/* /home/INdep/src/gpib/gpib.112/umigen.hxx */
/*
 * This file is automatically created
 * DO NOT EDIT
 */
#ifndef _GUMI_MSGS_HXX_
#define _GUMI_MSGS_HXX_
namespace INdepGpib100 {

  // 0 0


  /*
   * class GpibIfConf - Interface configuration
   */

  class GpibIfConf {
    u_int _fSc;	/* system controller	*/
    u_int _iPa;	/* primary interface address	*/
    u_int _fSa;	/* secondary interface address flag	*/
    u_int _iSa;	/* secondary interface address	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibIfConf&);
     friend xostream& xmarshal(xostream &xo, const GpibIfConf&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibIfConf&);
    #endif
    bool fSc() const { return _fSc!= 0 ; }
    GpibIfConf& fSc(bool __fSc) { _fSc=__fSc ; return *this; }
    u_int iPa() const { return _iPa; }
    GpibIfConf& iPa(u_int __iPa) { _iPa=__iPa ; return *this; }
    bool fSa() const { return _fSa!= 0 ; }
    GpibIfConf& fSa(bool __fSa) { _fSa=__fSa ; return *this; }
    u_int iSa() const { return _iSa; }
    GpibIfConf& iSa(u_int __iSa) { _iSa=__iSa ; return *this; }

    GpibIfConf():_fSc(false),_iPa(0),_fSa(false),_iSa(0) { }
    GpibIfConf(bool fSc,u_int iPa,bool fSa,u_int iSa):_fSc(fSc),_iPa(iPa),_fSa(fSa),_iSa(iSa) { }
    ~GpibIfConf() {}
  };
  // 0 0


  /*
   * class GpibTermOut - send termination descriptor
   */

  class GpibTermOut {
    u_int _chEos;	/* EOS byte for END	*/
    u_int _fEnd;	/* set END on last byte	*/
    u_int _fEoiEos;	/* set END on EOS	*/
    u_int _fEos7;	/* 7 bit EOS recognition	*/
    u_int _fHighSpeed;	/* use high-speed timing	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibTermOut&);
     friend xostream& xmarshal(xostream &xo, const GpibTermOut&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibTermOut&);
    #endif
    char chEos() const { return (char) _chEos; }
    GpibTermOut& chEos(char __chEos) { _chEos=__chEos ; return *this; }
    bool fEnd() const { return _fEnd!= 0 ; }
    GpibTermOut& fEnd(bool __fEnd) { _fEnd=__fEnd ; return *this; }
    bool fEoiEos() const { return _fEoiEos!= 0 ; }
    GpibTermOut& fEoiEos(bool __fEoiEos) { _fEoiEos=__fEoiEos ; return *this; }
    bool fEos7() const { return _fEos7!= 0 ; }
    GpibTermOut& fEos7(bool __fEos7) { _fEos7=__fEos7 ; return *this; }
    bool fHighSpeed() const { return _fHighSpeed!= 0 ; }
    GpibTermOut& fHighSpeed(bool __fHighSpeed) { _fHighSpeed=__fHighSpeed ; return *this; }

    GpibTermOut():_chEos(0),_fEnd(false),_fEoiEos(false),_fEos7(false),_fHighSpeed(false) { }
    GpibTermOut(char chEos,bool fEnd,bool fEoiEos,bool fEos7,bool fHighSpeed):_chEos(chEos),_fEnd(fEnd),_fEoiEos(fEoiEos),_fEos7(fEos7),_fHighSpeed(fHighSpeed) { }
    ~GpibTermOut() {}
  };
  // 0 0


  /*
   * class GpibTermIn - receive termination descriptor
   */

  class GpibTermIn {
    u_int _cEos;	/* EOS byte count	*/
    u_int _fDelEnd;	/* END message terminates transfer	*/
    u_int _fEos7;	/* 7 bit EOS recognition	*/
    u_int _mpchEos[3];	/* EOS characters	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibTermIn&);
     friend xostream& xmarshal(xostream &xo, const GpibTermIn&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibTermIn&);
    #endif
    u_int cEos() const { return _cEos; }
    GpibTermIn& cEos(u_int __cEos) { _cEos=__cEos ; return *this; }
    bool fDelEnd() const { return _fDelEnd!= 0 ; }
    GpibTermIn& fDelEnd(bool __fDelEnd) { _fDelEnd=__fDelEnd ; return *this; }
    bool fEos7() const { return _fEos7!= 0 ; }
    GpibTermIn& fEos7(bool __fEos7) { _fEos7=__fEos7 ; return *this; }
    char chEos(int i) const { return (char) _mpchEos[i]; }
    void chEos( int i, char __chEos) { _mpchEos[i]=__chEos ; }

    GpibTermIn():_cEos(0),_fDelEnd(true),_fEos7(false) { }
    GpibTermIn(u_int cEos,bool fDelEnd,bool fEos7):_cEos(cEos),_fDelEnd(fDelEnd),_fEos7(fEos7) { }
    ~GpibTermIn() {}
  };
  // 1 1


  /*
   * class GpibTrf - common transfer parameters
   */

  class GpibTrf:public Umi::UmiControl  {
    Umi::Time _tiTmo;
    u_int _iTrans;	/* transaction id	*/
    u_int _fLast;	/* last of transaction	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibTrf&);
     friend xostream& xmarshal(xostream &xo, const GpibTrf&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibTrf&);
    #endif
    const Umi::Time& tiTmo() const { return _tiTmo; }
    void tiTmo( const Umi::Time& __tiTmo) {_tiTmo=__tiTmo ; }
    u_int iTrans() const { return _iTrans; }
    GpibTrf& iTrans(u_int __iTrans) { _iTrans=__iTrans ; return *this; }
    bool fLast() const { return _fLast!= 0 ; }
    GpibTrf& fLast(bool __fLast) { _fLast=__fLast ; return *this; }

    GpibTrf(int e=0x1, size_t s=0);
    ~GpibTrf() {}
  };
  // 3 0


  /*
   * class GpibCmd - send commands request
   */

  class GpibCmd:public GpibTrf  {
   public:
     friend xistream& xunmarsh(xistream &xi, GpibCmd&);
     friend xostream& xmarshal(xostream &xo, const GpibCmd&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibCmd&);
    #endif

    GpibCmd(int e=0x3, size_t s=0);
    ~GpibCmd() {}
  };
  // 5 0


  /*
   * class GpibCwr - send data request
   */

  class GpibCwr:public GpibTrf  {
    GpibTermOut _termOut;
   public:
     friend xistream& xunmarsh(xistream &xi, GpibCwr&);
     friend xostream& xmarshal(xostream &xo, const GpibCwr&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibCwr&);
    #endif
    const GpibTermOut& termOut() const { return _termOut; }
    void termOut( const GpibTermOut& __termOut) {_termOut=__termOut ; }

    GpibCwr(int e=0x5, size_t s=0);
    ~GpibCwr() {}
  };
  // 8 1


  /*
   * class GpibCrd - receive data request
   */

  class GpibCrd:public Umi::UmiControl  {
    GpibTermIn _termIn;
    Umi::Time _tiTmo;
    u_int _cMax;	/* maximum transfer count read	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibCrd&);
     friend xostream& xmarshal(xostream &xo, const GpibCrd&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibCrd&);
    #endif
    const GpibTermIn& termIn() const { return _termIn; }
    void termIn( const GpibTermIn& __termIn) {_termIn=__termIn ; }
    const Umi::Time& tiTmo() const { return _tiTmo; }
    void tiTmo( const Umi::Time& __tiTmo) {_tiTmo=__tiTmo ; }
    u_int cMax() const { return _cMax; }
    GpibCrd& cMax(u_int __cMax) { _cMax=__cMax ; return *this; }

    GpibCrd(int e=0x8, size_t s=0);
    ~GpibCrd() {}
  };
  // 10 1


  /*
   * class GpibInit - Interface initialization
   */

  class GpibInit:public Umi::UmiControl  {
    GpibIfConf _ifConfig;
   public:
     friend xistream& xunmarsh(xistream &xi, GpibInit&);
     friend xostream& xmarshal(xostream &xo, const GpibInit&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibInit&);
    #endif
    const GpibIfConf& ifConfig() const { return _ifConfig; }
    void ifConfig( const GpibIfConf& __ifConfig) {_ifConfig=__ifConfig ; }

    GpibInit(int e=0x10, size_t s=0);
    ~GpibInit() {}
  };
  // 20 1


  /*
   * class GpibFini - Interface shutdown
   */

  class GpibFini:public Umi::UmiControl  {
   public:
     friend xistream& xunmarsh(xistream &xi, GpibFini&);
     friend xostream& xmarshal(xostream &xo, const GpibFini&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibFini&);
    #endif

    GpibFini(int e=0x20, size_t s=0);
    ~GpibFini() {}
  };
  // 40 1


  /*
   * class GpibResp - status response
   */

  class GpibResp:public Umi::UmiControl  {
    u_int _eErr;	/* result (error) code	*/
    u_int _iErr;	/* result (error) code, detailed	*/
    u_int _fStat;	/* status flags (StatE)	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibResp&);
     friend xostream& xmarshal(xostream &xo, const GpibResp&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibResp&);
    #endif
    u_int eErr() const { return _eErr; }
    GpibResp& eErr(u_int __eErr) { _eErr=__eErr ; return *this; }
    u_int iErr() const { return _iErr; }
    GpibResp& iErr(u_int __iErr) { _iErr=__iErr ; return *this; }
    u_int fStat() const { return _fStat; }
    GpibResp& fStat(u_int __fStat) { _fStat=__fStat ; return *this; }

    GpibResp(int e=0x40, size_t s=0);
    ~GpibResp() {}
  };
  // c0 0


  /*
   * class GpibRdResp - read response
   */

  class GpibRdResp:public GpibResp  {
    u_int _fDel;	/* message terminated	*/
    u_int _fDelEnd;	/* message terminated by END	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibRdResp&);
     friend xostream& xmarshal(xostream &xo, const GpibRdResp&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibRdResp&);
    #endif
    bool fDel() const { return _fDel!= 0 ; }
    GpibRdResp& fDel(bool __fDel) { _fDel=__fDel ; return *this; }
    bool fDelEnd() const { return _fDelEnd!= 0 ; }
    GpibRdResp& fDelEnd(bool __fDelEnd) { _fDelEnd=__fDelEnd ; return *this; }

    GpibRdResp(int e=0xc0, size_t s=0);
    ~GpibRdResp() {}
  };
  // 140 0


  /*
   * class GpibWrResp - write response
   */

  class GpibWrResp:public GpibResp  {
    u_int _cTrans;	/* bytes transmitted	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibWrResp&);
     friend xostream& xmarshal(xostream &xo, const GpibWrResp&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibWrResp&);
    #endif
    u_int cTrans() const { return _cTrans; }
    GpibWrResp& cTrans(u_int __cTrans) { _cTrans=__cTrans ; return *this; }

    GpibWrResp(int e=0x140, size_t s=0);
    ~GpibWrResp() {}
  };
  // 0 0


  /*
   * class GpibInfo - Interface information
   */

  class GpibInfo {
    u_int _eArch;	/* interface architecture id	*/
    u_int _cIntr;	/* hardware interrupt counter	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibInfo&);
     friend xostream& xmarshal(xostream &xo, const GpibInfo&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibInfo&);
    #endif
    u_int eArch() const { return _eArch; }
    GpibInfo& eArch(u_int __eArch) { _eArch=__eArch ; return *this; }
    u_int cIntr() const { return _cIntr; }
    GpibInfo& cIntr(u_int __cIntr) { _cIntr=__cIntr ; return *this; }

    GpibInfo():_eArch(0),_cIntr(0) { }
    GpibInfo(u_int eArch,u_int cIntr):_eArch(eArch),_cIntr(cIntr) { }
    ~GpibInfo() {}
  };
  // 200 1


  /*
   * class GpibInfoResp - info response
   */

  class GpibInfoResp:public Umi::UmiControl  {
    GpibInfo _info;
   public:
     friend xistream& xunmarsh(xistream &xi, GpibInfoResp&);
     friend xostream& xmarshal(xostream &xo, const GpibInfoResp&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibInfoResp&);
    #endif
    const GpibInfo& info() const { return _info; }
    void info( const GpibInfo& __info) {_info=__info ; }

    GpibInfoResp(int e=0x200, size_t s=0);
    ~GpibInfoResp() {}
  };
  // 400 1


  /*
   * class GpibIfc - send IFC
   */

  class GpibIfc:public Umi::UmiControl  {
    Umi::Time _ti;
   public:
     friend xistream& xunmarsh(xistream &xi, GpibIfc&);
     friend xostream& xmarshal(xostream &xo, const GpibIfc&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibIfc&);
    #endif
    const Umi::Time& ti() const { return _ti; }
    void ti( const Umi::Time& __ti) {_ti=__ti ; }

    GpibIfc(int e=0x400, size_t s=0);
    ~GpibIfc() {}
  };
  // 800 1


  /*
   * class GpibWait - wait for event
   */

  class GpibWait:public Umi::UmiControl  {
    Umi::Time _ti;
    u_int _fStat;	/* status flags (StatE)	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibWait&);
     friend xostream& xmarshal(xostream &xo, const GpibWait&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibWait&);
    #endif
    const Umi::Time& ti() const { return _ti; }
    void ti( const Umi::Time& __ti) {_ti=__ti ; }
    u_int fStat() const { return _fStat; }
    GpibWait& fStat(u_int __fStat) { _fStat=__fStat ; return *this; }

    GpibWait(int e=0x800, size_t s=0);
    ~GpibWait() {}
  };
  // 1000 1


  /*
   * class GpibRen - REN control
   */

  class GpibRen:public Umi::UmiControl  {
    u_int _fSet;	/* set/clr flag	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibRen&);
     friend xostream& xmarshal(xostream &xo, const GpibRen&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibRen&);
    #endif
    bool fSet() const { return _fSet!= 0 ; }
    GpibRen& fSet(bool __fSet) { _fSet=__fSet ; return *this; }

    GpibRen(int e=0x1000, size_t s=0);
    ~GpibRen() {}
  };
  // 2000 1


  /*
   * class GpibSta - status report
   */

  class GpibSta:public Umi::UmiControl  {
    u_int _iStb;	/* status byte	*/
    u_int _fMss;	/* summary status	*/
    u_int _fNew;	/* new summary status	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibSta&);
     friend xostream& xmarshal(xostream &xo, const GpibSta&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibSta&);
    #endif
    u_int iStb() const { return _iStb; }
    GpibSta& iStb(u_int __iStb) { _iStb=__iStb ; return *this; }
    bool fMss() const { return _fMss!= 0 ; }
    GpibSta& fMss(bool __fMss) { _fMss=__fMss ; return *this; }
    bool fNew() const { return _fNew!= 0 ; }
    GpibSta& fNew(bool __fNew) { _fNew=__fNew ; return *this; }

    GpibSta(int e=0x2000, size_t s=0);
    ~GpibSta() {}
  };
  // 4000 1


  /*
   * class GpibLnReq - lines status request
   */

  class GpibLnReq:public Umi::UmiControl  {
   public:
     friend xistream& xunmarsh(xistream &xi, GpibLnReq&);
     friend xostream& xmarshal(xostream &xo, const GpibLnReq&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibLnReq&);
    #endif

    GpibLnReq(int e=0x4000, size_t s=0);
    ~GpibLnReq() {}
  };
  // 8040 0


  /*
   * class GpibLnResp - line status response
   */

  class GpibLnResp:public GpibResp  {
    u_int _fCap;	/* cap flag	*/
    u_int _fStat;	/* stat flag	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibLnResp&);
     friend xostream& xmarshal(xostream &xo, const GpibLnResp&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibLnResp&);
    #endif
    u_int fCap() const { return _fCap; }
    GpibLnResp& fCap(u_int __fCap) { _fCap=__fCap ; return *this; }
    u_int fStat() const { return _fStat; }
    GpibLnResp& fStat(u_int __fStat) { _fStat=__fStat ; return *this; }

    GpibLnResp(int e=0x8040, size_t s=0);
    ~GpibLnResp() {}
  };
  // 10000 1


  /*
   * class GpibPeek - register read
   */

  class GpibPeek:public Umi::UmiControl  {
    u_int _iPort;	/* register number	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibPeek&);
     friend xostream& xmarshal(xostream &xo, const GpibPeek&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibPeek&);
    #endif
    u_int iPort() const { return _iPort; }
    GpibPeek& iPort(u_int __iPort) { _iPort=__iPort ; return *this; }

    GpibPeek(int e=0x10000, size_t s=0);
    ~GpibPeek() {}
  };
  // 30000 0


  /*
   * class GpibPoke - register write
   */

  class GpibPoke:public GpibPeek  {
    u_int _iVal;	/* register value	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibPoke&);
     friend xostream& xmarshal(xostream &xo, const GpibPoke&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibPoke&);
    #endif
    u_int iVal() const { return _iVal; }
    GpibPoke& iVal(u_int __iVal) { _iVal=__iVal ; return *this; }

    GpibPoke(int e=0x30000, size_t s=0);
    ~GpibPoke() {}
  };
  // 40000 1


  /*
   * class GpibNop - no operation
   */

  class GpibNop:public Umi::UmiControl  {
   public:
     friend xistream& xunmarsh(xistream &xi, GpibNop&);
     friend xostream& xmarshal(xostream &xo, const GpibNop&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibNop&);
    #endif

    GpibNop(int e=0x40000, size_t s=0);
    ~GpibNop() {}
  };
  // 80000 1


  /*
   * class GpibDeviceStatus - Device interface status
   */

  class GpibDeviceStatus:public Umi::UmiControl  {
    u_int _fStat;	/* status flag	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibDeviceStatus&);
     friend xostream& xmarshal(xostream &xo, const GpibDeviceStatus&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibDeviceStatus&);
    #endif
    u_int fStat() const { return _fStat; }
    GpibDeviceStatus& fStat(u_int __fStat) { _fStat=__fStat ; return *this; }

    GpibDeviceStatus(int e=0x80000, size_t s=0);
    ~GpibDeviceStatus() {}
  };
  // 180000 0


  /*
   * class GpibDeviceInput - device input data
   */

  class GpibDeviceInput:public GpibDeviceStatus  {
    u_int _fEND;	/* message terminated by END	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibDeviceInput&);
     friend xostream& xmarshal(xostream &xo, const GpibDeviceInput&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibDeviceInput&);
    #endif
    bool fEND() const { return _fEND!= 0 ; }
    GpibDeviceInput& fEND(bool __fEND) { _fEND=__fEND ; return *this; }

    GpibDeviceInput(int e=0x180000, size_t s=0);
    ~GpibDeviceInput() {}
  };
  // 280000 0


  /*
   * class GpibDeviceEvent - device event
   */

  class GpibDeviceEvent:public GpibDeviceStatus  {
    u_int _eEvent;	/* event enumerator	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibDeviceEvent&);
     friend xostream& xmarshal(xostream &xo, const GpibDeviceEvent&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibDeviceEvent&);
    #endif
    u_int eEvent() const { return _eEvent; }
    GpibDeviceEvent& eEvent(u_int __eEvent) { _eEvent=__eEvent ; return *this; }

    GpibDeviceEvent(int e=0x280000, size_t s=0);
    ~GpibDeviceEvent() {}
  };
  // 400000 1


  /*
   * class GpibDeviceOutput - device output data
   */

  class GpibDeviceOutput:public Umi::UmiControl  {
    u_int _fEND;	/* message terminated by END	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibDeviceOutput&);
     friend xostream& xmarshal(xostream &xo, const GpibDeviceOutput&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibDeviceOutput&);
    #endif
    bool fEND() const { return _fEND!= 0 ; }
    GpibDeviceOutput& fEND(bool __fEND) { _fEND=__fEND ; return *this; }

    GpibDeviceOutput(int e=0x400000, size_t s=0);
    ~GpibDeviceOutput() {}
  };
  // 800000 1


  /*
   * class GpibDeviceInitAck - device INIT ack
   */

  class GpibDeviceInitAck:public Umi::UmiControl  {
   public:
     friend xistream& xunmarsh(xistream &xi, GpibDeviceInitAck&);
     friend xostream& xmarshal(xostream &xo, const GpibDeviceInitAck&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibDeviceInitAck&);
    #endif

    GpibDeviceInitAck(int e=0x800000, size_t s=0);
    ~GpibDeviceInitAck() {}
  };
  // 1000000 1


  /*
   * class GpibDeviceStatusReport - device status report
   */

  class GpibDeviceStatusReport:public Umi::UmiControl  {
    u_int _fStb;	/* status byte	*/
    u_int _fSre;	/* srq enable	*/
    u_int _fForce;	/* force MAV	*/
   public:
     friend xistream& xunmarsh(xistream &xi, GpibDeviceStatusReport&);
     friend xostream& xmarshal(xostream &xo, const GpibDeviceStatusReport&);
    #ifndef _MM_NO_IOSTREAM
     friend std::ostream& operator<<(std::ostream &os, const GpibDeviceStatusReport&);
    #endif
    u_int fStb() const { return _fStb; }
    GpibDeviceStatusReport& fStb(u_int __fStb) { _fStb=__fStb ; return *this; }
    u_int fSre() const { return _fSre; }
    GpibDeviceStatusReport& fSre(u_int __fSre) { _fSre=__fSre ; return *this; }
    u_int fForce() const { return _fForce; }
    GpibDeviceStatusReport& fForce(u_int __fForce) { _fForce=__fForce ; return *this; }

    GpibDeviceStatusReport(int e=0x1000000, size_t s=0);
    ~GpibDeviceStatusReport() {}
  };

  inline bool isGpibTrf(const Umi::UmiControl *puc) { return ((puc->eType() & 0x1)==0x1); }
  extern GpibTrf* pGpibTrf(Umi::UmiControl *);
  inline const GpibTrf* pGpibTrf(const Umi::UmiControl *puc) { return pGpibTrf(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibCmd(const Umi::UmiControl *puc) { return ((puc->eType() & 0x3)==0x3); }
  extern GpibCmd* pGpibCmd(Umi::UmiControl *);
  inline const GpibCmd* pGpibCmd(const Umi::UmiControl *puc) { return pGpibCmd(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibCwr(const Umi::UmiControl *puc) { return ((puc->eType() & 0x5)==0x5); }
  extern GpibCwr* pGpibCwr(Umi::UmiControl *);
  inline const GpibCwr* pGpibCwr(const Umi::UmiControl *puc) { return pGpibCwr(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibCrd(const Umi::UmiControl *puc) { return ((puc->eType() & 0x8)==0x8); }
  extern GpibCrd* pGpibCrd(Umi::UmiControl *);
  inline const GpibCrd* pGpibCrd(const Umi::UmiControl *puc) { return pGpibCrd(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibInit(const Umi::UmiControl *puc) { return ((puc->eType() & 0x10)==0x10); }
  extern GpibInit* pGpibInit(Umi::UmiControl *);
  inline const GpibInit* pGpibInit(const Umi::UmiControl *puc) { return pGpibInit(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibFini(const Umi::UmiControl *puc) { return ((puc->eType() & 0x20)==0x20); }
  extern GpibFini* pGpibFini(Umi::UmiControl *);
  inline const GpibFini* pGpibFini(const Umi::UmiControl *puc) { return pGpibFini(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibResp(const Umi::UmiControl *puc) { return ((puc->eType() & 0x40)==0x40); }
  extern GpibResp* pGpibResp(Umi::UmiControl *);
  inline const GpibResp* pGpibResp(const Umi::UmiControl *puc) { return pGpibResp(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibRdResp(const Umi::UmiControl *puc) { return ((puc->eType() & 0xc0)==0xc0); }
  extern GpibRdResp* pGpibRdResp(Umi::UmiControl *);
  inline const GpibRdResp* pGpibRdResp(const Umi::UmiControl *puc) { return pGpibRdResp(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibWrResp(const Umi::UmiControl *puc) { return ((puc->eType() & 0x140)==0x140); }
  extern GpibWrResp* pGpibWrResp(Umi::UmiControl *);
  inline const GpibWrResp* pGpibWrResp(const Umi::UmiControl *puc) { return pGpibWrResp(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibInfoResp(const Umi::UmiControl *puc) { return ((puc->eType() & 0x200)==0x200); }
  extern GpibInfoResp* pGpibInfoResp(Umi::UmiControl *);
  inline const GpibInfoResp* pGpibInfoResp(const Umi::UmiControl *puc) { return pGpibInfoResp(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibIfc(const Umi::UmiControl *puc) { return ((puc->eType() & 0x400)==0x400); }
  extern GpibIfc* pGpibIfc(Umi::UmiControl *);
  inline const GpibIfc* pGpibIfc(const Umi::UmiControl *puc) { return pGpibIfc(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibWait(const Umi::UmiControl *puc) { return ((puc->eType() & 0x800)==0x800); }
  extern GpibWait* pGpibWait(Umi::UmiControl *);
  inline const GpibWait* pGpibWait(const Umi::UmiControl *puc) { return pGpibWait(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibRen(const Umi::UmiControl *puc) { return ((puc->eType() & 0x1000)==0x1000); }
  extern GpibRen* pGpibRen(Umi::UmiControl *);
  inline const GpibRen* pGpibRen(const Umi::UmiControl *puc) { return pGpibRen(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibSta(const Umi::UmiControl *puc) { return ((puc->eType() & 0x2000)==0x2000); }
  extern GpibSta* pGpibSta(Umi::UmiControl *);
  inline const GpibSta* pGpibSta(const Umi::UmiControl *puc) { return pGpibSta(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibLnReq(const Umi::UmiControl *puc) { return ((puc->eType() & 0x4000)==0x4000); }
  extern GpibLnReq* pGpibLnReq(Umi::UmiControl *);
  inline const GpibLnReq* pGpibLnReq(const Umi::UmiControl *puc) { return pGpibLnReq(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibLnResp(const Umi::UmiControl *puc) { return ((puc->eType() & 0x8040)==0x8040); }
  extern GpibLnResp* pGpibLnResp(Umi::UmiControl *);
  inline const GpibLnResp* pGpibLnResp(const Umi::UmiControl *puc) { return pGpibLnResp(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibPeek(const Umi::UmiControl *puc) { return ((puc->eType() & 0x10000)==0x10000); }
  extern GpibPeek* pGpibPeek(Umi::UmiControl *);
  inline const GpibPeek* pGpibPeek(const Umi::UmiControl *puc) { return pGpibPeek(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibPoke(const Umi::UmiControl *puc) { return ((puc->eType() & 0x30000)==0x30000); }
  extern GpibPoke* pGpibPoke(Umi::UmiControl *);
  inline const GpibPoke* pGpibPoke(const Umi::UmiControl *puc) { return pGpibPoke(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibNop(const Umi::UmiControl *puc) { return ((puc->eType() & 0x40000)==0x40000); }
  extern GpibNop* pGpibNop(Umi::UmiControl *);
  inline const GpibNop* pGpibNop(const Umi::UmiControl *puc) { return pGpibNop(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibDeviceStatus(const Umi::UmiControl *puc) { return ((puc->eType() & 0x80000)==0x80000); }
  extern GpibDeviceStatus* pGpibDeviceStatus(Umi::UmiControl *);
  inline const GpibDeviceStatus* pGpibDeviceStatus(const Umi::UmiControl *puc) { return pGpibDeviceStatus(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibDeviceInput(const Umi::UmiControl *puc) { return ((puc->eType() & 0x180000)==0x180000); }
  extern GpibDeviceInput* pGpibDeviceInput(Umi::UmiControl *);
  inline const GpibDeviceInput* pGpibDeviceInput(const Umi::UmiControl *puc) { return pGpibDeviceInput(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibDeviceEvent(const Umi::UmiControl *puc) { return ((puc->eType() & 0x280000)==0x280000); }
  extern GpibDeviceEvent* pGpibDeviceEvent(Umi::UmiControl *);
  inline const GpibDeviceEvent* pGpibDeviceEvent(const Umi::UmiControl *puc) { return pGpibDeviceEvent(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibDeviceOutput(const Umi::UmiControl *puc) { return ((puc->eType() & 0x400000)==0x400000); }
  extern GpibDeviceOutput* pGpibDeviceOutput(Umi::UmiControl *);
  inline const GpibDeviceOutput* pGpibDeviceOutput(const Umi::UmiControl *puc) { return pGpibDeviceOutput(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibDeviceInitAck(const Umi::UmiControl *puc) { return ((puc->eType() & 0x800000)==0x800000); }
  extern GpibDeviceInitAck* pGpibDeviceInitAck(Umi::UmiControl *);
  inline const GpibDeviceInitAck* pGpibDeviceInitAck(const Umi::UmiControl *puc) { return pGpibDeviceInitAck(const_cast<Umi::UmiControl*>(puc)); }
  inline bool isGpibDeviceStatusReport(const Umi::UmiControl *puc) { return ((puc->eType() & 0x1000000)==0x1000000); }
  extern GpibDeviceStatusReport* pGpibDeviceStatusReport(Umi::UmiControl *);
  inline const GpibDeviceStatusReport* pGpibDeviceStatusReport(const Umi::UmiControl *puc) { return pGpibDeviceStatusReport(const_cast<Umi::UmiControl*>(puc)); }

  #define MSG_GpibTrf 0x1
  #define MSG_GpibCmd 0x3
  #define MSG_GpibCwr 0x5
  #define MSG_GpibCrd 0x8
  #define MSG_GpibInit 0x10
  #define MSG_GpibFini 0x20
  #define MSG_GpibResp 0x40
  #define MSG_GpibRdResp 0xc0
  #define MSG_GpibWrResp 0x140
  #define MSG_GpibInfoResp 0x200
  #define MSG_GpibIfc 0x400
  #define MSG_GpibWait 0x800
  #define MSG_GpibRen 0x1000
  #define MSG_GpibSta 0x2000
  #define MSG_GpibLnReq 0x4000
  #define MSG_GpibLnResp 0x8040
  #define MSG_GpibPeek 0x10000
  #define MSG_GpibPoke 0x30000
  #define MSG_GpibNop 0x40000
  #define MSG_GpibDeviceStatus 0x80000
  #define MSG_GpibDeviceInput 0x180000
  #define MSG_GpibDeviceEvent 0x280000
  #define MSG_GpibDeviceOutput 0x400000
  #define MSG_GpibDeviceInitAck 0x800000
  #define MSG_GpibDeviceStatusReport 0x1000000

  class Gpib_gco0_Proto:public Umi::UmiProto {
   public:
    #ifndef _MM_NO_IOSTREAM
      xistream& unmarsh(xistream &, const Umi::UmiControl &, Umi::UmiControl *) const;
      xostream& marshal(xostream &, const Umi::UmiControl *) const ;
    #endif
    Gpib_gco0_Proto();
    ~Gpib_gco0_Proto();
  };

  class Gpib_gti0_Proto:public Umi::UmiProto {
   public:
    #ifndef _MM_NO_IOSTREAM
      xistream& unmarsh(xistream &, const Umi::UmiControl &, Umi::UmiControl *) const;
      xostream& marshal(xostream &, const Umi::UmiControl *) const ;
    #endif
    Gpib_gti0_Proto();
    ~Gpib_gti0_Proto();
  };

  class Gpib_gdi0_Proto:public Umi::UmiProto {
   public:
    #ifndef _MM_NO_IOSTREAM
      xistream& unmarsh(xistream &, const Umi::UmiControl &, Umi::UmiControl *) const;
      xostream& marshal(xostream &, const Umi::UmiControl *) const ;
    #endif
    Gpib_gdi0_Proto();
    ~Gpib_gdi0_Proto();
  };

  #define PROTID_gco0 "(/Int1:bool,Int1:u_int,Int1:bool,Int1:u_int/)(/Int1:char,Int1:bool,Int1:bool,Int1:bool,Int1:bool/)(/Int1:u_int,Int1:bool,Int1:bool,Int1:char/)(/Int4:u_int,Int1:bool/(//))((/Int4:u_int,Int1:bool/(//))//)((/Int4:u_int,Int1:bool/(//))//(/Int1:char,Int1:bool,Int1:bool,Int1:bool,Int1:bool/))(/Int4:u_int/(/Int1:u_int,Int1:bool,Int1:bool,Int1:char/),(//))(//(/Int1:bool,Int1:u_int,Int1:bool,Int1:u_int/))(//)(/Int4:u_int,Int4:u_int,Int4:u_int/)((/Int4:u_int,Int4:u_int,Int4:u_int/)/Int1:bool,Int1:bool/)((/Int4:u_int,Int4:u_int,Int4:u_int/)/Int4:u_int/)(/Int4:u_int,Int4:u_int/)(//(/Int4:u_int,Int4:u_int/))(//(//))(/Int4:u_int/(//))(/Int1:bool/)(/Int2:u_int,Int1:bool,Int1:bool/)(//)((/Int4:u_int,Int4:u_int,Int4:u_int/)/Int4:u_int,Int4:u_int/)"

  #define PROTID_gti0 "(/Int1:u_int/)((/Int1:u_int/)/Int1:u_int/)(//)"

  #define PROTID_gdi0 "(/Int4:u_int/)((/Int4:u_int/)/Int1:bool/)((/Int4:u_int/)/Int4:u_int/)(/Int1:bool/)(//)(/Int4:u_int,Int4:u_int,Int4:u_int/)"
}
#endif
/* /home/INdep/src/gpib/gpib.112/gca.hxx */

#ifndef _GPIB_GCA_HXX_
#define _GPIB_GCA_HXX_


#if (defined(MMSYS_linux) || defined(MMSYS_lkern) || defined (MMSYS_mingw))
#endif
#ifndef _MM_NO_IOSTREAM
#endif

namespace INdepGpib100 {


  namespace Umi=INdepUmi100;

  /*
   * interface chip,  possibly shared by different drivers
   */
  class GifCard:public Umi::UmiCard, private Regs {

    enum {
      MAX_IREG=4,
      MAX_REG=24,
      MAX_AUX=8,
      F_ALL = 0xff,
      _END
    };

    unsigned char _mpis[MAX_IREG];		/* interrupt stats	*/

   #ifdef DEBUG
    int _mpisRef[MAX_IREG];		/* reference stats	*/
   #endif

    unsigned char _mpIst[MAX_REG];	/* interrupt status	*/
    unsigned char _mpIsh[MAX_REG];	/* interrupt shadows	*/
    unsigned char _mpor[MAX_REG];	/* output register copy	*/
    unsigned char _mpoa[MAX_AUX];	/* output aux regs	*/

    static const bool mpfIma[CNT_GCA][16];

    int _eArch;				/* controller architecture	*/
    bool _fSc;				/* system controller		*/
    bool _fIntr;			/* enable interrupt 		*/
    int _eInfo;				/* for print			*/
    Counter _ctIntr; 

    void setInfo(int e) { _eInfo=e; }

    int i_read(int i) { return _mpIst[i]=(unsigned char)(read(i)); }

    /* get the interrupt status shadow registers */
    bool isIma(int i) { return mpfIma[_eArch][i]; }
    bool isIntr() { return _fIntr; }

   public:
    bool isGood() const { return Regs::isGood(); }

    int eArch() { return _eArch; }
    bool isSc() { return _fSc; }
    bool isiGpib() const { return _eArch == GCA_72010; }
    bool isQuick() const { return _eArch == GCA_GP5130; }
    int reset(bool fSc);
    void is_clear() {  for (int i=0;i<MAX_IREG;i++) _mpis[i]=0; }
    char *print( char *, int ) const;

   #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<< ( std::ostream &os, const GifCard &);
   #endif
   #ifdef DEBUG
    bool getIsChange();
   #endif


    /* non-data/non-auxiliary register access */
    void outreg( int iReg,  int iVal )  { 
      _mpor[iReg] = (unsigned char)iVal;
      if (isIntr() || !isIma(iReg) ) write(iReg, _mpor[iReg] );
    }

    /* data output/input */
    void outdat( int iVal ) { write(IGC_DIO, iVal); }
    int indat( ) { return read(IGC_DIO); }

    /* auxiliary functions accessed via auxmod */
    void outaux( int iAux, int iMask, int iVal ) {
      write( IGC_AUXMOD , iAux | (_mpoa[iAux >> 5] = (_mpoa[iAux >> 5] & ~iMask) | iVal));
    }

    int r_test(int iReg, int f ) { return read(iReg) & f; }
    int r_read(int iReg ) { return read(iReg); }
    int isCic( ) { return (read(IGC_ADSTS) & IGC_ADSTS_CIC); }

    /* get the interface status */
    StatE getStatE( );			

    /* 
     * updating interrupt functions 
     *
     *********************************************
     *    WARNING: might reset active INT pin    *
     *********************************************
     */
    bool isIntReg(int i) {  return mpfIma[_eArch][i]; }

    //int i_test( int iReg, int f ) {
    //  return ((_mpIsh[iReg] |= i_read(iReg)) & f);
    //}

    int i_check(int i) {
      register int f;
      _mpIsh[i] |= (f=_mpIst[i]=(unsigned char)read(i));  /* get the current state, reset source */
      return (_mpor[i] & f ) ;
    }


    int i_get( int iReg, int f ) {
      int i;
      i=(_mpIsh[iReg] |= i_read(iReg));
      _mpIsh[iReg] &= ~f;
      return i & f;
    }

    int i_cur( int iReg, int f ) {
      int i;
      _mpIsh[iReg] |= (i=i_read(iReg));
      return i & f;
    }


    int i_spin( int iReg, int fTest, int fClear, int cSpin ) {
      int i;
      while (!(i=((_mpIsh[iReg] |= i_read(iReg)) & fTest)) && cSpin--);
      _mpIsh[iReg] &= ~fClear;
      return i;
    }

    bool isIrqReason();

    bool isIrq() { 
      int i;
      _mpIsh[IGC_I2] |= (i=i_read(IGC_I2));
      return (( i & 0x80 ) != 0);
    } 


    int interrupt(int iCtx);
    int timer(int iCtx, const Umi::Time &);


    /* 
     * non-updating interrupt functions 
     */
    int i_was( int iReg, int f ) { return _mpIsh[iReg] & f; }
    int i_last( int iReg, int f ) { return _mpIst[iReg] & f; }
    int i_clr( int iReg, int f ) {
      int i;
      i=_mpIsh[iReg];
      _mpIsh[iReg] &= ~f;
      return i & f;
    }
    int i_set( int iReg, int f ) { return _mpIsh[iReg] |= f; }

    /* commands given via auxmod */
    void outcmd( int iVal ) { write(IGC_AUXMOD, iVal); }
    void outeos( int iVal ) { write(IGC_EOS1, iVal); }
    void outspoll( int iVal ) { write(IGC_SP, iVal); }

    int cNecClk() { return 8; };	/* clock divider */
    int fIntPol() { return 0; };	/* int polarity  */

    int recvData(Umi::UmiPutBuf *pup, const GpibTermIn &gti, int cSpin);
    int sendData(Umi::UmiGetBuf *pug, const GpibTermOut &gto, int cSpin);
   
    GifCard(intptr_t addr, int eArch, int fIntr);
    ~GifCard();
  };

}
#endif

/* /home/INdep/src/gpib/gpib.112/gco.hxx */

#ifndef _GPIB_NECIF_HXX_
#define _GPIB_NECIF_HXX_

#ifndef _MM_NO_IOSTREAM
#endif

namespace INdepGpib100 {

  namespace Umi=INdepUmi100;

  /*
   * NEC 7210 based interface - Controller and Device 
   */
  class GifControl:public Umi::UmiDriver, private SmM {

   private: 
    /*
     * states
     */
    enum {
      S_POFF,
      S_IDLE,	/* awaiting new command */
      S_CWR,	/* conducted write	*/
      S_CRD,	/* conducted read	*/
      S_CMD,	/* commands		*/
      S_IFC, 	/* send IFC		*/
      S_REN, 	/* send REN false	*/
      S_WSRQ, 	/* wait for SRQ		*/
      CNT_STATES
    };



    /* message handlers */
    int msgInit(int iCtx, const Umi::UmiMsgDown *pmd );
    int msgFini(int iCtx, const Umi::UmiMsgDown *pmd);


    int msgIfc(int iCtx, Umi::Pmd pmd);
    int msgRen(int iCtx, Umi::Pmd pmd);
    int msgCmd(int iCtx, Umi::Pmd pmd);
    int msgWr(int iCtx, Umi::Pmd pmd);
    int msgRd(int iCtx, Umi::Pmd pmd);
    int msgSta(int iCtx, Umi::Pmd pmd);
    int msgWait(int iCtx, Umi::Pmd pmd);



    /* state handlers */
    int completeSend(int iCtx, const GpibTrf *pgm);
    int hCRD(int iCtx);
    int hCWR(int iCtx);
    int hCMD(int iCtx);
    int hIFC(int iCtx);
    int hREN(int iCtx);
    int hWSRQ(int iCtx);
    int switchState(int iCtx);

    int gts();
    void setMode( const GpibTermIn &gti);
    void setMode( const GpibTermOut &gto);

    int cSpinCmd() { return 1; }	/* default spin count for commands */
    int cSpinDwr() { return 1; }	/* default spin count for writes   */
    int cSpinDrd() { return 1; }	/* default spin count for reads    */

    /*
     * interface configuration
     */
    GpibIfConf _gic;

    /*
     * request  parameters
     */
    Umi::Pmd _pmd;		/* current downstream handled	*/
    Umi::Time _tiNext;		/* (T), next timeout		*/
    int _eErr;
    int _iErr;
    void setErr(int eErr, int iErr) { _eErr=eErr, _iErr=iErr; }
    void clrErr() { _eErr=0, _iErr=0; }
    int isErr() { return _eErr; }

    /*
     * IFC, REN variables
     */
    int _fTimerDone;
    
    /*
     * Transfer parameters
     */
    size_t _cTrans;
    size_t _cTransPrev;

    /*
     * READ parameters
     */
    Umi::Pmu _pmuNext;		/* next read response message 	*/
    size_t _cRdRem;		/* remaining read count	      	*/
    GpibTermIn _gti;		/* input terminator		*/
    Umi::Time _tiTmo;		/* (t), per byte timeout	*/
    

    /*
     * enter fail state
     */
    int failed(int iCtx, Umi::Pmd pmd, int eErr, int iErr, int (GifControl::*pf) (int), int eState) ;
    int drop(int iCtx, Umi::Pmd pmd);

    /*
     * responses
     */
    int putUpResp(int iCtx, Umi::Pmu pmu, GpibResp *pmg);		/* complete state 	*/	
    int putUpResp( int iCtx, const Umi::UmiMsgDown *pmd , ErrE eErr, int iErr=0  );	/* immediate */

    /*
     * transactions 
     */
    int _iTransCur;		/* current transaction 		*/
    int _iTransFailed;		/* == iTransCur after fail	*/


    const GpibIfConf &gic() { return _gic; }
    GifCard *pca() { return static_cast<GifCard*>(Umi::UmiDriver::pca()); }

    /*
     * response utilities
     * release umd, allocate GpibMsgXXX, setup result values and putU
     */
    bool isTransFailed(int i) { return i == _iTransFailed; }
    void failTransaction() { _iTransFailed = _iTransCur; }
    int isTransCur(int i) { return i == _iTransCur; }
    void setTransCur(int i) { _iTransFailed = ~(_iTransCur = i); }


    /* statistics */
    enum StacE {
      STAC_CMD_OUT,
      STAC_BYTE_OUT,
      STAC_BYTE_IN,
      CNT_STAC
    };
    int _mpcStac[CNT_STAC];
    void incStac(StacE e, int c=1) { _mpcStac[e]+=c; }
    void setStac(StacE e, int i) { _mpcStac[e]=i; }


    void debug( const char *sz );

   public:

    /* debug print */
 #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<< ( std::ostream &os, const GifControl &);
 #endif

    const Umi::UmiProto *pup(int iChan);		/* proto desc for channel */

    int cStacMax() const { return CNT_STAC; }
    int cStac(int e) const { return _mpcStac[e]; }

    char *print( char *, int );

    /* 
     * message put -- Note: timer() must has been called 
     * once before using putDown() the first time
     */ 
    int putDown(int iCtx, Umi::Pmd pmd, int iChan);

    int open(int iChan);
    void close(int iChan);

    /* interupt handler */
    int interrupt(int iCtx);
    void backenable(int iCtx, int iChan);

    /* timer handler */
    int timer(int iCtx, const Umi::Time &ti);

    GifControl(GifCard *pca);
    virtual ~GifControl();
  };
}
#endif

/* /home/INdep/src/gpib/gpib.112/gdi.hxx */

#ifndef _GPIB_GDI_HXX_
#define _GPIB_GDI_HXX_


namespace INdepGpib100 {
  namespace Umi=INdepUmi100;

  /*
   * NEC 7210 based interface - Device 
   */
  class GifDevice:public Umi::UmiDriver, private SmM {

   private: 
    /*
     * states
     */
    enum {
      S_POFF,
      S_BAV,	
      S_BRQ,	
      S_DCAS,
      S_DONE,	
      S_IDLE,
      S_READ,	
      S_RESP,	
      S_WAIT,	
      CNT_STATES
    };

    int cSpinWr() { return 1; }	/* default spin count for commands */
    int cSpinRd() { return 1; }	/* default spin count for writes   */

    /* variables */
    const Umi::UmiMsgUp *_pmuNext;		/* next upstream message	*/
    const Umi::UmiMsgDown *_pmdCur;		/* current output message	*/
    int _eEvent;				/* last event seen		*/
    int _eMsg;					/* last message seen		*/
    int _eStLast;				/* last state 			*/
    int eEvent() const { return _eEvent; }
    int eMsg() const { return _eMsg; }
    int eStLast() const { return _eStLast; }
    int setState(int e) { _eStLast=eStatePrev(e); return e; }


    /*
     * interface configuration
     */
    GpibIfConf _gic;
    const GpibIfConf &gic() { return _gic; }

    GifCard *pca() const { return static_cast<GifCard*>(Umi::UmiDriver::pca()); }


    /* statistics */
    enum StacE {
      STAC_BYTE_IN,
      CNT_STAC
    };
    int _mpcStac[CNT_STAC];
    void incStac(StacE e, int i) { _mpcStac[e]+=i; }
    void setStac(StacE e, int i) { _mpcStac[e]=i; }


    void debug( const char *sz );
    int putUpDone( int iCtx, const Umi::UmiMsgDown *pmd , ErrE eErr, int iErr = 0 );
    bool putUpEvent(int iCtx, int eEvent);

    bool sendData(int iCtx, Umi::Pmd pmd);
    void sendDataCancel(int iCtx);
    bool recvData(int iCtx);

    Umi::UmiDriver::DownE msgInit(int iCtx, Umi::Pmd pmd );
    Umi::UmiDriver::DownE msgFini(int iCtx, Umi::Pmd pmd );

    /*
     * state machine 
     */
    bool isBavGet() { return (pca()->i_was(IGC_I1, IGC_I1_DI) || pca()->i_was(IGC_I1, IGC_I1_DET)); } 
    bool isBrq() { return (pca()->i_was(IGC_I1, IGC_I1_DO)!=0); } 
    typedef void (GifDevice::* PFhState) (int iCtx, int, Umi::Pmd pmd);		

    void hPOFF(int iCtx, int eMsg, Umi::Pmd pmd );
    void hIDLE(int iCtx, int eMsg, Umi::Pmd pmd );
    void hDCAS(int iCtx, int eMsg, Umi::Pmd pmd );
    void hREAD(int iCtx, int eMsg, Umi::Pmd pmd );
    void hWAIT(int iCtx, int eMsg, Umi::Pmd pmd );
    void  hBRQ(int iCtx, int eMsg, Umi::Pmd pmd );
    void  hBAV(int iCtx, int eMsg, Umi::Pmd pmd );
    void hRESP(int iCtx, int eMsg, Umi::Pmd pmd );
    void hDONE(int iCtx, int eMsg, Umi::Pmd pmd );
    static PFhState mphs[CNT_STATES];
    void doSm(int iCtx, int eMsg, Umi::Pmd pmd, int eEvent );
    Umi::Pmd defaultDown(Umi::Pmd pmd );

   public:

    /* debug print */
   #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<< ( std::ostream &os, const GifDevice &);
   #endif

    int cStacMax() const { return CNT_STAC; }
    int cStac(int e) const { return _mpcStac[e]; }

    char *print( char *, int );
    const Umi::UmiProto *pup(int iChan);		/* proto desc for channel */

    /* 
     * message put -- Note: timer() must have been called 
     * once before using putDown() the first time
     */ 
    int putDown(int iCtx, Umi::Pmd pmd, int iChan);

    int open(int iChan);
    void close(int iChan);

    void backenable(int iCtx, int iChan);

    /* interupt handler */
    int interrupt(int iCtx);

    /* timer handler */
    int timer(int iCtx, const Umi::Time &ti);

    GifDevice(GifCard *pca);
    virtual ~GifDevice();
  };

}
#endif

/* /home/INdep/src/gpib/gpib.112/gti.hxx */

#ifndef _GPIB_GTI_HXX_
#define _GPIB_GTI_HXX_

#ifndef _MM_NO_IOSTREAM
#endif

namespace INdepGpib100 {
  namespace Umi=INdepUmi100;

  /*
   * NEC 7210 based interface - Controller and Device 
   */
  class GifTest:public Umi::UmiDriver, private SmM {

   private: 
    enum { S_IDLE, S_INIT };

    /* message handlers */
    int msgNop(int iCtx, const Umi::UmiMsgDown *pmd );
    int msgPeek(int iCtx, Umi::Pmd pmd);
    int msgPoke(int iCtx, Umi::Pmd pmd);


    /* variables */
    int _cIntr;				/* interrupt counter		*/
    GifCard *pca() { return static_cast<GifCard*>(Umi::UmiDriver::pca()); }
    int putBackDone( int iCtx, const Umi::UmiMsgDown *pmd , ErrE e, int iErr =0 );
   public:

    /* debug print */
 #ifndef _MM_NO_IOSTREAM
    friend std::ostream &operator<< ( std::ostream &os, const GifTest &);
 #endif
    char *print( char *, int );
    const Umi::UmiProto *pup(int iChan);		/* proto desc for channel */

    /* 
     * message put -- Note: timer() must has been called 
     * once before using putDown() the first time
     */ 
    int putDown(int iCtx, Umi::Pmd pmd, int iChan);

    int open(int iChan);
    void close(int iChan);

    /* interupt handler */
    int interrupt(int iCtx);

    /* timer handler */
    int timer(int iCtx, const Umi::Time &ti);

    GifTest(GifCard *pca);
    virtual ~GifTest();
  };


}
#endif

